<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Customer_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with Customer Roles and User Info.
     *
     * @since 1.0.0
     */
    class MM_REST_Customer_V1_Controller extends WC_REST_Controller {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of MM_REST_Customer_V1_Controller.
         *
         * @var MM_REST_Customer_V1_Controller
         */
        private static $_instance;


        /**
         * Endpoint namespace.
         *
         * @var string
         */
        protected $namespace = 'mm/v1';

        /**
         * Route base.
         *
         * @var string
         */
        protected $rest_base = 'customers';


        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_Customer_V1_Controller constructor.
         *
         * @since 1.0.0
         * @access public
         */
        public function __construct() {

            
            // Fires when preparing to serve an API request.
            add_action("rest_api_init", array($this, "register_routes"));
            
           
        }

        /**
         * Register routes for ManageMore roles API.
         *
         * @since 1.0.0
         * @access public
         */
        public function register_routes() {


            register_rest_route($this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_user'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                ),
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'update_user'),
                    'permission_callback' => array($this, 'update_user_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/custid/(?P<cust_id>[a-z0-9_]*)' , array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_user'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                ),
            ));


            register_rest_route($this->namespace, '/' . $this->rest_base . '/email/(?P<user_email>\S+)' , array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_user'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/login/(?P<user_login>\S+)' , array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_user'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/product_pricing/(?P<id>\d+)' , array(
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'update_product_customer_pricing'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/members', array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_users_by_status'),
                    'permission_callback' => array($this, 'get_user_permissions_check'),
                    'args' => array(
                        'status' => array(
                            'validate_callback' => function($param, $request, $key) {
                                return in_array($param, array('pending', 'rejected', 'approved', 'inactive','disabled','expired_1','expired_2','expired_3','all'));
                            }
                        ),
                        'per_page' => array(
                            'validate_callback' => function($param, $request, $key) {
                                return is_numeric($param);
                            },
                            'default' => 50,
                            'maximum' => 100,
                        ),
                        'page' => array(
                            'validate_callback' => function($param, $request, $key) {
                                return is_numeric($param);
                            },
                            'default' => 1,
                        ),
                        'search' => array(  
                            'validate_callback' => function($param, $request, $key) {
                                return is_string($param);
                            },
                            'default' => '',
                        ),               

                        'registered_since' => array(
                            'default' => null,
                            'validate_callback' => function($param, $request, $key) {
                                // Checks if the date is valid and is in 'Y-m-d' format
                                $date = DateTime::createFromFormat('Y-m-d', $param);
                                return $date && $date->format('Y-m-d') === $param;
                            },
                            'sanitize_callback' => 'sanitize_text_field',
                        ),


                    ),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/membership/(?P<id>\d+)', array(
               array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'change_membership_status'),
                    'permission_callback' => array($this, 'update_user_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/membership/batch', array(
                'methods' => WP_REST_Server::EDITABLE,
                'callback' => array($this, 'change_membership_status_batch'),
                'permission_callback' => array($this, 'update_user_permissions_check'),
            ));

        }

 
        /**
         * Check if a given request has access to read the user.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function get_user_permissions_check($request) {
        	
            $user_id = $request['id'];
            $user = get_userdata($user_id);

            if ($user && !wc_rest_check_user_permissions( 'read', $user_id)) {
                return new WP_Error('mm_rest_cannot_view', __('Sorry, you cannot view this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }

        /**
         * Check if a given request has access to update the user.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function update_user_permissions_check($request) {

            $user_id = $request['id'];
            $user = get_userdata($user_id);


            if ($user && !wc_rest_check_user_permissions( 'edit',  $user_id)) {
                return new WP_Error('mm_rest_cannot_edit', __('Sorry, you are not allowed to edit this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }

        /**
         * Get one user.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function get_user($request) {

           $user_id = isset($request['id']) ? $request['id'] : '';
 
            if (!empty($user_id)) {
               //Get user by Id
               $result = $this->get_customer_info($user_id);
               return new WP_REST_Response($result, 200);
            }
            elseif (isset($request['cust_id'])){
              //Get user by MM CustId (as a parameter, this is cust_id, stored as mm_field_CID)
              $mm_field_CID = strtoupper($request['cust_id']);
              $users = get_users(array(
                   'meta_key' => 'mm_field_CID',
                   'meta_value' => $mm_field_CID
                    ));
            
              if(empty($users)){
             	 $result = array();
              }        
              else {
              	 $user_id = $users[0]->ID;
                $result = $this->get_customer_info($user_id);
             	
              }
              return new WP_REST_Response($result, 200);
            }
            elseif (isset($request['user_email'])){
              //Get user by email
              $user_email = urldecode($request['user_email']);
           	  $user = get_user_by('email',$user_email);
            
              if(empty($user)){
             	 $result = array();
              }        
              else {
              	 $user_id = $user->ID;
                $result = $this->get_customer_info($user_id);
              }
              return new WP_REST_Response($result, 200);
            }
            elseif (isset($request['user_login'])){
              //Get user by email
              $user_login = $request['user_login'];
           	  $user = get_user_by('login',$user_login);
            
              if(empty($user)){
             	 $result = array();
              }        
              else {
              	 $user_id = $user->ID;
                $result = $this->get_customer_info($user_id);
              }
              return new WP_REST_Response($result, 200);
            }



            return new WP_Error('mm_rest_cannot_view', __('User not found.', 'managemore-connector'), array('status' => rest_authorization_required_code()));

        }

        /**
         * Update a user
         *
         * @param WP_REST_Request        
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function update_user($request) {
        	
        	   global $woocommerce;

            $user_id = isset($request['id']) ? $request['id'] : '';

            if (!empty($user_id)) {

                $user = get_userdata($user_id);
                if ($user==false){
                	 //Id not found - return blank response
                	 $result = array();
                   return new WP_REST_Response($result, 200);
                }
                
                //Check if passed user id is contributor or above (can edit posts) Do not allow update via MM REST (should only be for Customers, e.g. Subscribers)
                if (user_can( $user_id, 'edit_posts' )){
                  //This will return NON-CUSTOMER role only.                	
                  $result = $this->get_customer_info($user_id);
                  return new WP_REST_Response($result, 200);
               }
               
                //Check if "disable" is passed.  If so, set the flag accordingly
                if (isset($request['disable_user'])){
                	$disable_user = $request['disable_user'];
                  update_user_meta($user_id,'mm_disable_user',$disable_user);
                }
                
                //Check if member_action passed.  If so, perform the action
                if (isset($request['member_action'])){
                	 $result = $this->mm_perform_membership_action($user_id,$request['member_action']);
                   // Check for error
                   if (is_wp_error($result)) {
                       return $result;
                   }
                }

                //If role passed, set role (cannot set to admin)
                $role = $request['role'];
                if (! empty( $role ) && $role !== 'administrator' ){
                	 if ($GLOBALS['wp_roles']->is_role( $role )){
                       $user->set_role($role);
                	 }
                } 
                
                //If custid passed, update meta value
                //As a parameter, it is cust_id, but stored as mm_field_CID
                if (isset($request['cust_id'])){
                   $mm_field_CID = strtoupper($request['cust_id']) ;
                	 update_user_meta($user_id,'mm_field_CID',$mm_field_CID);
                }
                
                //If email passed, update it (this will send an email to old address by default)
                if (isset($request['email']) && $request['email']!=''){
                   $email = $request['email'];
                   if(is_email($email) == false){
                   	  $error_string = $email . ' is an invalid email address';
                   	  return new WP_Error('mm_rest_cannot_update',$error_string, array('status' => rest_authorization_required_code()));
                   }
                   $user_data = wp_update_user( array( 'ID' => $user_id, 'user_email' => $email ) );
                   if (is_wp_error($user_data)){
                   	  $error_string = $user_data->get_error_message();
                   	  return new WP_Error('mm_rest_cannot_update',$error_string, array('status' => rest_authorization_required_code()));
                   }
                   //Update WooCommerce email (which is separate)
                	 update_user_meta($user_id,'billing_email',$email);
                }

                //If Company Name passed, update it
                //passed as company_name, stored as billing_company
                if (isset($request['company_name'])){
                   $company_name = $request['company_name'] ;
                	 update_user_meta($user_id,'billing_company',$company_name);
                }

                //If first_name or last_name passed, update them
                if (isset($request['first_name']) || isset($request['last_name'])){
                   $first_name = $request['first_name'] ;
                   $last_name = $request['last_name'] ;
                	 update_user_meta($user_id,'first_name',$first_name);
                	 update_user_meta($user_id,'last_name',$last_name);
                	 //Also update billing first name and last name
                	 update_user_meta($user_id,'billing_first_name',$first_name);
                	 update_user_meta($user_id,'billing_last_name',$last_name);
                }


                //If nickname is passed, update it
                if (isset($request['display_name'])){
                   $display_name = $request['display_name'] ;
                   $user_data = wp_update_user( array( 'ID' => $user_id, 'display_name' => $display_name ) );
                }

               
                //If Billing Info Passed, update it. Note: This is an all or none update, since the address is a whole entity
                //If billing_address_1 passed, update the entire address
                if (isset($request['billing_address_1'])){
                   //Set variables from request.  Set to blank if not passed
                   $billing_address_1 = $request['billing_address_1'] ;
                   $billing_address_2 = isset($request['billing_address_2']) ? $request['billing_address_2'] : '';
                   $billing_city = isset($request['billing_city']) ? $request['billing_city'] : '';
                   $billing_state = isset($request['billing_state']) ? $request['billing_state'] : '';
                   $billing_postcode = isset($request['billing_postcode']) ? $request['billing_postcode'] : '';
                   $billing_country = isset($request['billing_country']) ? $request['billing_country'] : '';
                   //Update meta values
                	 update_user_meta($user_id,'billing_address_1',$billing_address_1);
                	 update_user_meta($user_id,'billing_address_2',$billing_address_2);
                	 update_user_meta($user_id,'billing_city',$billing_city);
                	 update_user_meta($user_id,'billing_state',$billing_state);
                	 update_user_meta($user_id,'billing_postcode',$billing_postcode);
                	 update_user_meta($user_id,'billing_country',$billing_country);
                }

                //If phone passed, update it
                //passed as phone, stored as billing_phone
                if (isset($request['phone'])){
                   $phone = $request['phone'] ;
                	 update_user_meta($user_id,'billing_phone',$phone);
                }

                //If Shipping Company Name passed, update it
                if (isset($request['shipping_company_name'])){
                   $shipping_company_name = $request['shipping_company_name'] ;
                	 update_user_meta($user_id,'shipping_company',$shipping_company_name);
                }

                //If first_name or last_name passed, update them
                if (isset($request['shipping_first_name']) || isset($request['shipping_last_name'])){
                   $shipping_first_name = $request['shipping_first_name'] ;
                   $shipping_last_name = $request['shipping_last_name'] ;
                	 update_user_meta($user_id,'shipping_first_name',$shipping_first_name);
                	 update_user_meta($user_id,'shipping_last_name',$shipping_last_name);
                }


                //If Shipping Info Passed, update it. Note: This is an all or none update, since the address is a whole entity
                //If shipping_address_1 passed, update the entire address
                if (isset($request['shipping_address_1'])){
                   //Set variables from request.  Set to blank if not passed
                   $shipping_address_1 = $request['shipping_address_1'] ;
                   $shipping_address_2 = isset($request['shipping_address_2']) ? $request['shipping_address_2'] : '';
                   $shipping_city = isset($request['shipping_city']) ? $request['shipping_city'] : '';
                   $shipping_state = isset($request['shipping_state']) ? $request['shipping_state'] : '';
                   $shipping_postcode = isset($request['shipping_postcode']) ? $request['shipping_postcode'] : '';
                   $shipping_country = isset($request['shipping_country']) ? $request['shipping_country'] : '';
                   //Update meta values
                	 update_user_meta($user_id,'shipping_address_1',$shipping_address_1);
                	 update_user_meta($user_id,'shipping_address_2',$shipping_address_2);
                	 update_user_meta($user_id,'shipping_city',$shipping_city);
                	 update_user_meta($user_id,'shipping_state',$shipping_state);
                	 update_user_meta($user_id,'shipping_postcode',$shipping_postcode);
                	 update_user_meta($user_id,'shipping_country',$shipping_country);
                }

                //If new password passed, update it (will send email by default)
                if (isset($request['new_password'])){
                   $new_password = $request['new_password'];
                   $user_data = wp_update_user( array( 'ID' => $user_id, 'user_pass' => $new_password ) );
                }
                
                //If customer pricing passed, update it
                if (isset($request['customer_pricing'])){
                	$customer_pricing = $request['customer_pricing'];  //should be a semicolon/pipe delimited string of price rules ("productid;price|productid;price")
                	MM_Connector_Functions::update_customer_pricing($customer_pricing,$user_id,1);  
                }
                //If indirect_tax_exemptions passed, update it
                if (isset($request['indirect_tax_exemptions'])){
                	 $indirect_tax_exemptions = $request['indirect_tax_exemptions'];  //should be a pipe delimited string of indirect taxes exempted ("id|id|id")
                	 //Note: The ids passed are ManageMore Ids, we need to look up and store the ids of the Indirect Tax Types (post ids)
                	 $indirect_tax_type_exemptions = $this->convert_tax_exemptions($indirect_tax_exemptions);
                	 update_user_meta($user_id,'indirect_tax_exemptions',$indirect_tax_type_exemptions);
                }

                //If balance field passed, update it and other statistical fields
                //Note: Most of these fields are stored using the econnect nomenclature of mm_field_CODE (the ones that can be used with CRM)
                //1.0.2 - Status added
                //3.0.4 - Since we now integrate with Ultimate Member, we store dates as yyyy/mm/dd to conform with their format
                //        However, ManageMore stores and passes/receives dates as mm/dd/yyyy - we did not change this.
                if (isset($request['mm_field_B'])){
 

                   $original_date = $request['mm_field_LPD'];  
                   $mm_field_LPD = '';  // Initialize with blank value
                   // Check if original date is not empty
                   if (!empty($original_date)) {
                      // Create DateTime object from original date
                      $date_obj = date_create_from_format('m/d/Y', $original_date);
                      // If date object is not false, the date string was in a correct format
                      if($date_obj !== false) {
                          // Format date to "mm/dd/yyyy" format
                          $mm_field_LPD = $date_obj->format('Y/m/d');
                      }
                   }               

                   $original_date = $request['mm_field_L1X'];  
                   $mm_field_L1X = '';  // Initialize with blank value
                   // Check if original date is not empty
                   if (!empty($original_date)) {
                      // Create DateTime object from original date
                      $date_obj = date_create_from_format('m/d/Y', $original_date);
                      // If date object is not false, the date string was in a correct format
                      if($date_obj !== false) {
                          // Format date to "mm/dd/yyyy" format
                          $mm_field_L1X = $date_obj->format('Y/m/d');
                      }
                   }               
                   $original_date = $request['mm_field_L2X'];  
                   $mm_field_L2X = '';  // Initialize with blank value
                   // Check if original date is not empty
                   if (!empty($original_date)) {
                      // Create DateTime object from original date
                      $date_obj = date_create_from_format('m/d/Y', $original_date);
                      // If date object is not false, the date string was in a correct format
                      if($date_obj !== false) {
                          // Format date to "mm/dd/yyyy" format
                          $mm_field_L2X = $date_obj->format('Y/m/d');
                      }
                   }               
                   $original_date = $request['mm_field_L3X'];  
                   $mm_field_L3X = '';  // Initialize with blank value
                   // Check if original date is not empty
                   if (!empty($original_date)) {
                      // Create DateTime object from original date
                      $date_obj = date_create_from_format('m/d/Y', $original_date);
                      // If date object is not false, the date string was in a correct format
                      if($date_obj !== false) {
                          // Format date to "mm/dd/yyyy" format
                          $mm_field_L3X = $date_obj->format('Y/m/d');
                      }
                   }               
 
 
                   $mm_field_B = $request['mm_field_B'] ;                	
                 	 update_user_meta($user_id,'mm_field_B',$mm_field_B);
                   $mm_field_LAR = $request['mm_field_LAR'] ;                	
                 	 update_user_meta($user_id,'mm_field_LAR',$mm_field_LAR);
                   $mm_field_L1N = $request['mm_field_L1N'] ;                	
                 	 update_user_meta($user_id,'mm_field_L1N',$mm_field_L1N);
                   $mm_field_L2N = $request['mm_field_L2N'] ;                	
                 	 update_user_meta($user_id,'mm_field_L2N',$mm_field_L2N);
                   $mm_field_L3N = $request['mm_field_L3N'] ;                	
                 	 update_user_meta($user_id,'mm_field_L3N',$mm_field_L3N);

                 	 //the values for these fields is set above, to convert the date format
                 	 update_user_meta($user_id,'mm_field_LPD',$mm_field_LPD);
                 	 update_user_meta($user_id,'mm_field_L1X',$mm_field_L1X);  
                 	 update_user_meta($user_id,'mm_field_L2X',$mm_field_L2X);
                 	 update_user_meta($user_id,'mm_field_L3X',$mm_field_L3X);

                   $mm_field_TXC = $request['mm_field_TXC'] ;                	
                 	 update_user_meta($user_id,'mm_field_TXC',$mm_field_TXC);
                   $mm_field_ITX = $request['mm_field_ITX'] ;                	
                 	 update_user_meta($user_id,'mm_field_ITX',$mm_field_ITX);
                   $mm_field_STAT = $request['mm_field_STAT'] ;                	
                 	 update_user_meta($user_id,'mm_field_STAT',$mm_field_STAT);
                   $mm_field_OAP = $request['mm_field_OAP'] ;                	
                 	 update_user_meta($user_id,'mm_field_OAP',$mm_field_OAP);
                   $mm_field_CLMT = $request['mm_field_CLMT'] ;                	
                 	 update_user_meta($user_id,'mm_field_CLMT',$mm_field_CLMT);
                   $mm_field_TXID = $request['mm_field_TXID'] ;                	
                 	 update_user_meta($user_id,'mm_field_TXID',$mm_field_TXID);
                   $mm_field_FXID = $request['mm_field_FXID'] ;                	
                 	 update_user_meta($user_id,'mm_field_FXID',$mm_field_FXID);
                 	 
                 	 $timestamp = current_time('timestamp',0);
                 	 update_user_meta($user_id,'mm_last_sync',$timestamp); //last sync of balance
                 	 
                   /* Check if we need to auto-clear the pending payment timestamp.
                   *  This holds the timetamp of when a WEBPMT sku is purchased (payment is made)
                   *  This value should be cleared when the payment is imported, but if that communication
                   *  does not happen correctly, the value would never get cleared and prevent future payments
                   *  This logic will auto-clear the value if the last payment date on record is on or after the
                   *  pending payment. So it should clear it on the next sync after payment was processed.
                   */
                 	 $pending_payment_timestamp = get_user_meta($user_id, 'mm_payment_pending', true);
                   if (!empty($mm_field_LPD) && !empty($pending_payment_timestamp)) {
                    	  $last_payment_date_time = $mm_field_LPD . ' 23:59:59'; //assume end of day
                       // Convert yyyy/mm/dd format to Unix timestamp
                       $lpd_timestamp = DateTime::createFromFormat('Y/m/d H:i:s', $last_payment_date_time)->getTimestamp();
                       // Compare the dates
                       if ($lpd_timestamp >= $pending_payment_timestamp) {
                          // The Last Payment Date is on or after the Pending Payment date
                          // Clear it
                          update_user_meta($user_id,'mm_payment_pending','');
                       }
                   }
                }
                
                if (isset($request['clear_payment_pending'])){
                 	 update_user_meta($user_id,'mm_payment_pending',''); 
                }

                $result = $this->get_customer_info($user_id);

                return new WP_REST_Response($result, 200);

            }

            return new WP_Error('mm_rest_cannot_update', __('No user provided.', 'managemore-connector'), array('status' => rest_authorization_required_code()));

        }


        /**
         * Update customer pricing for a particular product
         * This is used for the MM REST request route (mm/v1/customers/product_pricing/<id>)
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function update_product_customer_pricing($request) {
        	
           $product_id = isset($request['id']) ? $request['id'] : '';      	

        	  if (!empty($product_id)) {
        	  	
                //If customer pricing passed, update it
                if (isset($request['customer_pricing'])){
                	$customer_pricing = $request['customer_pricing'];  //should be a semicolon/pipe delimited string of price rules ("id;price;method;startdate;enddate|id;price;method;startdate;enddate")
                	MM_Connector_Functions::update_customer_pricing($customer_pricing,$product_id,2);  
                }
        	  	
                $result = array("id"=>$product_id);

                return new WP_REST_Response($result, 200);
       	  	
        	  	
        	  }
        	
        	  return new WP_Error('mm_rest_cannot_update', __('No product provided.', 'managemore-connector'), array('status' => rest_authorization_required_code())); 
        	
        } 

        

        /**
         * Retrieves users of a particular status from Ultimate Member.
         * 
         * This REST API endpoint returns a list of users based on their account 
         * status in the Ultimate Member plugin. The statuses can be 'pending', 
         * 'rejected', 'approved', or 'inactive'. If no status is provided in the 
         * request, it defaults to 'pending'. The endpoint supports pagination and 
         * returns a defined number of users per page (default is 50, maximum is 100).
         *
         * The response includes each user's id, first_name, last_name, email_address, 
         * all WooCommerce address fields, user registration date and time, as well as 
         * relevant user meta fields pertaining to their Ultimate Member account status.
         * It also includes the total count of users having the queried status.
         *
         * @param WP_REST_Request $request The request object containing optional 
         * parameters 'status', 'per_page', and 'page'.
         * @return WP_REST_Response The response object containing user data and 
         * total user count, or an error message.
         */
        public function get_users_by_status($request) {
            // Prepare the response array
            $response = array();

            // Ultimate Member statuses (and MM)
            $status_um = array(
                'pending' => 'awaiting_admin_review',
                'rejected' => 'rejected',
                'approved' => 'approved',
                'inactive' => 'inactive',
                'disabled' => 'disabled',
                'expired_1' => 'expired_1',
                'expired_2' => 'expired_2',
                'expired_3' => 'expired_3',
                'all' => 'all',
            );
            
            $status_name = array(
                'awaiting_admin_review' => 'Pending',
                'rejected' => 'Rejected',
                'approved' => 'Approved',
                'inactive' => 'Inactive',
            );

            // Get the status from the request, defaulting to 'pending' if it's not provided
            $status = $request->get_param('status') ? $request->get_param('status') : 'pending';

            // Map the provided status to the corresponding Ultimate Member status
            $status = isset($status_um[$status]) ? $status_um[$status] : $status_um['pending'];

            // Get the per_page from the request, defaulting to 50 if it's not provided
            $per_page = $request->get_param('per_page') ? intval($request->get_param('per_page')) : 50;
            // Restrict per_page to a maximum of 100
            $per_page = min($per_page, 100);

            // Get the page from the request, defaulting to 1 if it's not provided
            $page = $request->get_param('page') ? intval($request->get_param('page')) : 1;

            // Calculate the offset
            $offset = ($page - 1) * $per_page;
            
            //Get Search value
            $search = $request->get_param('search');
            
            // Get the 'registered_since' parameter from the request, defaulting to null if it's not provided
            $registered_since_str = $request->get_param('registered_since');
            $registered_since = null; // Initialize with null value

           // If 'registered_since' string is not empty, try to create a DateTime object from it
           if (!empty($registered_since_str)) {
              $registered_since = DateTime::createFromFormat('Y-m-d', $registered_since_str);

              // If the creation failed, return an error
              if ($registered_since === false) {
                  $registered_since = null; // Ignore if invalid
              }
           }

            // Base query parameters
            $query_params = array(
                'role__not_in' => array('administrator'),
                'search' => "*{$search}*",
                'search_columns' => array('user_login', 'user_nicename', 'user_email')
            );

            // Add the registered_since condition if it was provided
            if ($registered_since !== null) {
                $query_params['date_query'] = array(
                    array(
                        'column' => 'user_registered',
                        'after' => $registered_since->format('Y-m-d')
                    )
                );
            }

            // Based on the status, modify the query parameters 
            switch($status) {
                case 'all':
                    // If the status is 'all', no changes are required to the query parameters
                    break;
                case 'disabled':
                    // Only consider users whose account status is 'approved' and are disabled
                    $query_params = array_merge($query_params, array(
                        'meta_query' => array(
                            'relation' => 'AND',
                            array(
                                'key' => 'mm_disable_user',
                                'value' => '1'
                            ),
                            array(
                                'key' => 'account_status',
                                'value' => 'approved'
                            ),
                        ),
                    ));
                    break;

                case 'expired_1':
                case 'expired_2':
                case 'expired_3':
                    $license_num = str_replace("expired_", "", $status); // Get number of license
                    $today = date('Y/m/d'); // Get today's date in Ymd format
                    // Only consider users whose account status is 'approved' and the license is expired
                    $query_params = array_merge($query_params, array(
                        'meta_query' => array(
                            'relation' => 'AND',
                            array(
                                'relation' => 'OR',
                                array(
                                    'key' => 'mm_field_L'.$license_num.'X',
                                    'compare' => 'NOT EXISTS' // Check if the key does not exist
                                ),
                                array(
                                    'key' => 'mm_field_L'.$license_num.'X',
                                    'value' => $today,
                                    'compare' => '<',
                                ),
                                array(
                                    'key' => 'mm_field_L'.$license_num.'X',
                                    'value' => '',
                                    'compare' => '='
                                ),
                            ),
                            array(
                                'key' => 'account_status',
                                'value' => 'approved'
                            ),
                        ),
                    ));
                    break;
                default:
                    // For any other status, set the meta_key and meta_value based on the status
                    $query_params = array_merge($query_params, array(
                        'meta_key' => 'account_status',
                        'meta_value' => $status
                    ));
                    break;
            }


            //Get total records for our query. (paging is not included here)
            if ($page == 1){
               $total_args = array_merge($query_params, array(
                      'fields' => 'ID', // Just get the user IDs, we don't need the full user objects here
               ));
               $total_users = count(get_users($total_args)); // Count the total users with the provided status
            }

            //Add paging to the query
            $query_params = array_merge($query_params, array(
                'number' => $per_page,
                'offset' => $offset,
                ));


            //Get Users
            $users = get_users($query_params);
            $user_count = count($users);
            
            //Get Image URLs (UM stores the images in a standard place, based on the user id)
            $upload_dir = wp_upload_dir();


            // Loop through each pending user
            foreach ($users as $user) {

               $user_id = $user->ID;

               //Set Full URL for images to return (UM only stores the file name)
               $image_name=get_user_meta($user->ID, 'mm_field_L1I', true);
               if (!empty($image_name)){
                  $mm_license_1_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
               }
               else{
               	$mm_license_1_image_url = '';
               }
               $image_name=get_user_meta($user->ID, 'mm_field_L2I', true);
               if (!empty($image_name)){
                  $mm_license_2_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
               }
               else{
               	$mm_license_2_image_url = '';
               }
               $image_name=get_user_meta($user->ID, 'mm_field_L3I', true);
               if (!empty($image_name)){
                  $mm_license_3_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
               }
               else{
               	$mm_license_3_image_url = '';
               }
               
               //Get the license expiration dates and convert to mm/dd/yyyy
               //UM stores dates as yyyy/mm/dd
               $original_date = get_user_meta($user_id, 'mm_field_L1X', true);  
               $mm_field_L1X = '';  // Initialize with blank value
               // Check if original date is not empty
               if (!empty($original_date)) {
                   // Create DateTime object from original date
                   $date_obj = date_create_from_format('Y/m/d', $original_date);
                   // If date object is not false, the date string was in a correct format
                   if($date_obj !== false) {
                       // Format date to "mm/dd/yyyy" format
                       $mm_field_L1X = $date_obj->format('m/d/Y');
                   }
               }               
               $original_date = get_user_meta($user_id, 'mm_field_L2X', true);  
               $mm_field_L2X = '';  // Initialize with blank value
               // Check if original date is not empty
               if (!empty($original_date)) {
                   // Create DateTime object from original date
                   $date_obj = date_create_from_format('Y/m/d', $original_date);
                   // If date object is not false, the date string was in a correct format
                   if($date_obj !== false) {
                       // Format date to "mm/dd/yyyy" format
                       $mm_field_L2X = $date_obj->format('m/d/Y');
                   }
               }               
               $original_date = get_user_meta($user_id, 'mm_field_L3X', true);  
               $mm_field_L3X = '';  // Initialize with blank value
               // Check if original date is not empty
               if (!empty($original_date)) {
                   // Create DateTime object from original date
                   $date_obj = date_create_from_format('Y/m/d', $original_date);
                   // If date object is not false, the date string was in a correct format
                   if($date_obj !== false) {
                       // Format date to "mm/dd/yyyy" format
                       $mm_field_L3X = $date_obj->format('m/d/Y');
                   }
               }               
              
               // Get the user registration date and time
               $user_registered = $user->user_registered;
               // Create DateTime object from the registration date and time
               $datetime = new DateTime($user_registered);
               // Format the date and time
               $registered_date = $datetime->format('m/d/Y'); // outputs date as mm/dd/yyyy
               $registered_time = $datetime->format('H:i:s'); // outputs time as hh:mm:ss (24 hour format)

               //Get Membership Status, convert to nice string
               $member_status = $status_name[get_user_meta($user_id, 'account_status', true)];
               
               //Check if user is disabled for checkout
               $mm_disable_user = get_user_meta($user_id, 'mm_disable_user', true);

                // Prepare the user data
                $user_data = array(
                    'id' => $user_id,
                    'email_address' => $user->user_email,
                    'username' => $user->user_login,
                    'member_status' => $member_status,
                    'mm_disable_user' => $mm_disable_user,
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    // Get the user's WooCommerce fields
                    'billing_first_name' => get_user_meta($user_id, 'billing_first_name', true),
                    'billing_last_name' => get_user_meta($user_id, 'billing_last_name', true),
                    'billing_company' => get_user_meta($user_id, 'billing_company', true),
                    'billing_address_1' => get_user_meta($user_id, 'billing_address_1', true),
                    'billing_address_2' => get_user_meta($user_id, 'billing_address_2', true),
                    'billing_city' => get_user_meta($user_id, 'billing_city', true),
                    'billing_state' => get_user_meta($user_id, 'billing_state', true),
                    'billing_postcode' => get_user_meta($user_id, 'billing_postcode', true),
                    'billing_country' => get_user_meta($user_id, 'billing_country', true),
                    'billing_phone' => get_user_meta($user_id, 'billing_phone', true),
                    'shipping_first_name' => get_user_meta($user_id, 'shipping_first_name', true),
                    'shipping_last_name' => get_user_meta($user_id, 'shipping_first_name', true),
                    'shipping_company' => get_user_meta($user_id, 'shipping_company', true),
                    'shipping_address_1' => get_user_meta($user_id, 'shipping_address_1', true),
                    'shipping_address_2' => get_user_meta($user_id, 'shipping_address_2', true),
                    'shipping_city' => get_user_meta($user_id, 'shipping_city', true),
                    'shipping_state' => get_user_meta($user_id, 'shipping_state', true),
                    'shipping_postcode' => get_user_meta($user_id, 'shipping_postcode', true),
                    'shipping_country' => get_user_meta($user_id, 'shipping_country', true),
                    'shipping_phone' => get_user_meta($user_id, 'shipping_phone', true),
                    // Get the user's ManageMore fields
                    'license_1_number' => get_user_meta($user_id, 'mm_field_L1N', true),
                    'license_1_exp' => $mm_field_L1X,
                    'license_1_image' => $mm_license_1_image_url,
                    'license_2_number' => get_user_meta($user_id, 'mm_field_L2N', true),
                    'license_2_exp' => $mm_field_L2X,
                    'license_2_image' => $mm_license_2_image_url,
                    'license_3_number' => get_user_meta($user_id, 'mm_field_L3N', true),
                    'license_3_exp' => $mm_field_L3X,
                    'license_3_image' => $mm_license_3_image_url,
                    'state_tax_id' => get_user_meta($user_id, 'mm_field_TXID', true),
                    'federal_tax_id' => get_user_meta($user_id, 'mm_field_FXID', true),
                    //Date and Time Registered
                    'registered_date' => $registered_date,
                    'registered_time' => $registered_time,
                );

                // Add the user data to the response array
                $response[] = $user_data;
            }

            if ($page == 1){
               // Prepare the response
               $response_data = array(
                   'total_users' => $total_users, //only include the total in the first batch.
                   'user_count' => $user_count,
                   'users' => $response,
               );
            }
            else {
               // Prepare the response
               $response_data = array(
                   'user_count' => $user_count,
                   'users' => $response,
               );
            }

            // Return the response array
            return new WP_REST_Response($response_data, 200);
        }

        /**
         * A private method to perform membership actions.
         *
         * @param int $user_id - The ID of the user to perform action on.
         * @param string $action - The action to be performed.
         * @return int|WP_Error - Returns the user ID if successful, or WP_Error if not.
         */
        private function mm_perform_membership_action($user_id, $action) {

            if ($user_id == 0 || user_can( $user_id, 'administrator' ) ){
                return new WP_Error('mm_rest_invalid_user', 'Invalid user for membership action.', array('status' => 400));
            }


            // Check if the action is valid
            if (in_array($action, ['activate', 'deactivate', 'approve', 'reject', 'pending', 'email','remove'])) {
                um_fetch_user($user_id);
                switch ($action) {
                    case 'activate':
                    case 'approve':
                        UM()->user()->approve();
                        break;

                    case 'deactivate':
                        UM()->user()->deactivate();
                        break;

                    case 'reject':
                        UM()->user()->reject();
                        break;

                    case 'pending':
                        UM()->user()->pending();
                        break;

                    case 'email':
                        $user_data = get_userdata($user_id);
                        $user_email = $user_data->user_email;
                        UM()->mail()->send($user_email, 'approved_email');
                        break;
                    case 'remove':
                        require_once(ABSPATH.'wp-admin/includes/user.php' );
                        wp_delete_user( $user_id );
                        break;
                }
            } else {
                // Invalid action
                return new WP_Error('mm_rest_invalid_action', 'Invalid action parameter provided.', array('status' => 400));
            }

            // If no error, return user_id
            return $user_id;
        }

        /**
         * Changes the membership status of a user.
         * The function accepts the following actions:
         * 'activate'   - Activates the user's account.
         * 'deactivate' - Deactivates the user's account.
         * 'approve'    - Approves the user's account.
         * 'reject'     - Rejects the user's account.
         * 'pending'    - Marks the user's account as pending.
         * 'email'      - Resends the activation email to the user.
         *
         * @param WP_REST_Request $request Full data about the request.
         * @return WP_Error|WP_REST_Response
         */
        public function change_membership_status($request) {

            if (!MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) {
                // Ultimate Member plugin is not installed
                return new WP_Error('um_not_installed', 'The Ultimate Member plugin is not installed or active.', array('status' => 400));
            }

            $user_id = $request['id'];
            
           
            $action = $request->get_param('action');
            
            // Perform the action
            $result = $this->mm_perform_membership_action($user_id, $action);

            // Check for error
            if (is_wp_error($result)) {
                return $result;
            }

            // Success
            $response = array(
                'id' => $user_id,
                'status' => 'success',
            );
            return new WP_REST_Response($response, 200);
        }

        /**
         * Change membership status of multiple users in a batch.
         * 
         * The function accepts the following actions:
         * 'activate'   - Activates the user's account.
         * 'deactivate' - Deactivates the user's account.
         * 'approve'    - Approves the user's account.
         * 'reject'     - Rejects the user's account.
         * 'pending'    - Marks the user's account as pending.
         * 'email'      - Resends the activation email to the user.
         *
         * @param WP_REST_Request $request - Full details about the request
         * @return WP_Error|WP_REST_Response
         */
        public function change_membership_status_batch(WP_REST_Request $request) {
            // Request body should have the structure {"activate": [{"id":1},{"id":2}], "deactivate": [{"id":3},{"id":4}], "approve": [{"id":5},{"id":6}], "reject": [{"id":7},{"id":8}], "pending": [{"id":9},{"id":10}], "email": [{"id":11},{"id":12}]}
            $body = $request->get_json_params();
            $response = array();

            foreach ($body as $action => $users) {
                foreach ($users as $user) {

                    // Perform the action
                    $status_response = $this->mm_perform_membership_action($user['id'], $action);

                    // If the response is a WP_Error, record the error message
                    if (is_wp_error($status_response)) {
                        $response[$action][] = array('id' => $status_request['id'], 'status' => 'failed', 'message' => $status_response->get_error_message());
                    } else { // Otherwise, record success
                        $response[$action][] = array('id' => $status_request['id'], 'status' => 'success');
                    }
                }
            }

            return new WP_REST_Response($response, 200);
        }
 

        private function get_customer_info($user_id){
        	
        	
        	  $customer_info=array();
        	  
           $user = get_userdata($user_id);
           if ($user ==false){
           	  $customer_info = array();
           	  return $customer_info;
           }

           //Check if passed user id is contributor or above (can edit posts) Do not allow update via MM REST (should only be for Customers, e.g. Subscribers)
           if (user_can( $user_id, 'edit_posts' )){
           	   //only return role, to indicate that user does exist, but is not a customer
               $customer_info = array('id' => $user_id,
                                      'role'=>'NON-CUSTOMER',
                                      );
               return $customer_info;
           }

           //Get Image URLs (UM stores the images in a standard place, based on the user id)
           $upload_dir = wp_upload_dir();  //this was missing - fixed 4.5.3

           //Set Full URL for images to return (UM only stores the file name)
           $image_name=get_user_meta($user->ID, 'mm_field_L1I', true);
           if (!empty($image_name)){
              $mm_license_1_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
           }
           else{
           	$mm_license_1_image_url = '';
           }
           $image_name=get_user_meta($user->ID, 'mm_field_L2I', true);
           if (!empty($image_name)){
              $mm_license_2_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
           }
           else{
           	$mm_license_2_image_url = '';
           }
           $image_name=get_user_meta($user->ID, 'mm_field_L3I', true);
           if (!empty($image_name)){
              $mm_license_3_image_url = $upload_dir['baseurl'] . '/ultimatemember/' . $user->ID . '/' . $image_name;
           }
           else{
           	$mm_license_3_image_url = '';
           }
           
           //Get the license expiration dates and convert to mm/dd/yyyy
           //UM stores dates as yyyy/mm/dd
           $original_date = get_user_meta($user_id, 'mm_field_L1X', true);  
           $mm_field_L1X = '';  // Initialize with blank value
           // Check if original date is not empty
           if (!empty($original_date)) {
               // Create DateTime object from original date
               $date_obj = date_create_from_format('Y/m/d', $original_date);
               // If date object is not false, the date string was in a correct format
               if($date_obj !== false) {
                   // Format date to "mm/dd/yyyy" format
                   $mm_field_L1X = $date_obj->format('m/d/Y');
               }
           }               
           $original_date = get_user_meta($user_id, 'mm_field_L2X', true);  
           $mm_field_L2X = '';  // Initialize with blank value
           // Check if original date is not empty
           if (!empty($original_date)) {
               // Create DateTime object from original date
               $date_obj = date_create_from_format('Y/m/d', $original_date);
               // If date object is not false, the date string was in a correct format
               if($date_obj !== false) {
                   // Format date to "mm/dd/yyyy" format
                   $mm_field_L2X = $date_obj->format('m/d/Y');
               }
           }               
           $original_date = get_user_meta($user_id, 'mm_field_L3X', true);  
           $mm_field_L3X = '';  // Initialize with blank value
           // Check if original date is not empty
           if (!empty($original_date)) {
               // Create DateTime object from original date
               $date_obj = date_create_from_format('Y/m/d', $original_date);
               // If date object is not false, the date string was in a correct format
               if($date_obj !== false) {
                   // Format date to "mm/dd/yyyy" format
                   $mm_field_L3X = $date_obj->format('m/d/Y');
               }
           }               



           //Get most vaalues from meta
           //1.0.2 Status added
           $mm_field_CID = get_user_meta($user_id,'mm_field_CID',true);
           $mm_field_STAT = get_user_meta($user_id,'mm_field_STAT',true);
           $first_name = get_user_meta($user_id,'billing_first_name',true);
           $last_name = get_user_meta($user_id,'billing_last_name',true);
           $company_name = get_user_meta($user_id,'billing_company',true);
           $billing_address_1 = get_user_meta($user_id,'billing_address_1',true);
           $billing_address_2 = get_user_meta($user_id,'billing_address_2',true);
           $billing_city = get_user_meta($user_id,'billing_city',true);
           $billing_state = get_user_meta($user_id,'billing_state',true);
           $billing_postcode = get_user_meta($user_id,'billing_postcode',true);
           $billing_country = get_user_meta($user_id,'billing_country',true);
           $shipping_first_name = get_user_meta($user_id,'shipping_first_name',true);
           $shipping_last_name = get_user_meta($user_id,'shipping_last_name',true);
           $shipping_company_name = get_user_meta($user_id,'shipping_company',true);
           $shipping_address_1 = get_user_meta($user_id,'shipping_address_1',true);
           $shipping_address_2 = get_user_meta($user_id,'shipping_address_2',true);
           $shipping_city = get_user_meta($user_id,'shipping_city',true);
           $shipping_state = get_user_meta($user_id,'shipping_state',true);
           $shipping_postcode = get_user_meta($user_id,'shipping_postcode',true);
           $shipping_country = get_user_meta($user_id,'shipping_country',true);
           $phone = get_user_meta($user_id,'billing_phone',true);
           $mm_last_login = get_user_meta($user_id,'mm_last_login',true);  //last login timestamp
           if ($mm_last_login > 0 ){
              $last_login_date = date('m/d/Y',$mm_last_login);
              $last_login_time = date('H:i:s',$mm_last_login);
           }
           else{
              $last_login_date = '';
              $last_login_time = '';
           }
           $date_registered = date('m/d/Y',strtotime($user->user_registered));
           $mm_last_ip = get_user_meta($user_id,'mm_last_login_ip',true);  //last login ip address
           $mm_last_browser = get_user_meta($user_id,'mm_last_login_browser',true); //last login browser info
           $mm_disable_user = get_user_meta($user_id,'mm_disable_user',true);
           $member_status = get_user_meta($user_id,'account_status',true);  //Ultimate Member Status

           $customer_info = array('id' => $user_id,
                           'login_id'=>$user->user_login,
                           'cust_id'=>$mm_field_CID,
                           'cust_status'=>$mm_field_STAT,
                           'member_status' => $member_status,
                           'role'=>implode(', ', $user->roles),
                           'first_name'=>$first_name,
                           'last_name'=>$last_name,
                           'company_name'=>$company_name,
                           'display_name'=>$user->display_name,
                           'email'=>$user->user_email,
                           'phone'=>$phone,
                           'billing_address_1'=>$billing_address_1,
                           'billing_address_2'=>$billing_address_2,
                           'billing_city'=>$billing_city,
                           'billing_state'=>$billing_state,
                           'billing_postcode'=>$billing_postcode,
                           'billing_country'=>$billing_country,
                           'shipping_first_name'=>$shipping_first_name,
                           'shipping_last_name'=>$shipping_last_name,
                           'shipping_company_name'=>$shipping_company_name,
                           'shipping_address_1'=>$shipping_address_1,
                           'shipping_address_2'=>$shipping_address_2,
                           'shipping_city'=>$shipping_city,
                           'shipping_state'=>$shipping_state,
                           'shipping_postcode'=>$shipping_postcode,
                           'shipping_country'=>$shipping_country,
                           'disable_user'=>$mm_disable_user,
                           'license_1_number' => get_user_meta($user_id, 'mm_field_L1N', true),
                           'license_1_exp' => $mm_field_L1X,
                           'license_1_image' => $mm_license_1_image_url,
                           'license_2_number' => get_user_meta($user_id, 'mm_field_L2N', true),
                           'license_2_exp' => $mm_field_L2X,
                           'license_2_image' => $mm_license_2_image_url,
                           'license_3_number' => get_user_meta($user_id, 'mm_field_L3N', true),
                           'license_3_exp' => $mm_field_L3X,
                           'license_3_image' => $mm_license_3_image_url,
                           'state_tax_id' => get_user_meta($user_id, 'mm_field_TXID', true),
                           'federal_tax_id' => get_user_meta($user_id, 'mm_field_FXID', true),
                           'last_login_date'=>$last_login_date,
                           'last_login_time'=>$last_login_time,
                           'date_registered'=>$date_registered,
                           'last_login_ip'=>$mm_last_ip,
                           'last_login_browser'=>$mm_last_browser,
                           );
                           
           return $customer_info;
        	
        }

        //convert pipe delimited indirect tax id string (mm ids) to indirect tax type (postid) string
        private function convert_tax_exemptions($indirect_tax_exemptions){

            $manageMoreIDs = explode('|', $indirect_tax_exemptions);

            // Fetch all posts and their ManageMore IDs
            $args = array(
                'post_type' => 'mm-indirect-tax-type', // Replace with your actual post type
                'posts_per_page' => -1, // Fetch all posts
                'fields' => 'ids', // Only get post IDs to improve performance
            );

            $query = new WP_Query($args);
            $mapping = [];

            if ($query->have_posts()) {
                foreach ($query->posts as $postId) {
                    // Get the ManageMore ID stored in post meta
                    $mmRecId = get_post_meta($postId, 'mm_rec_id', true);
                    // Create a mapping of mmRecId to postId
                    $mapping[$mmRecId] = $postId;
                }
            }

            // Now convert the ManageMore ID string to website tax post ID string using the mapping
            $websiteTaxPostIDs = array_map(function ($id) use ($mapping) {
                return $mapping[$id] ?? null; // Return null or a default value if not found
            }, $manageMoreIDs);

            // Remove any null values
            $websiteTaxPostIDs = array_filter($websiteTaxPostIDs);

            // Convert the website tax post IDs back to a pipe-delimited string
            $websiteTaxPostIDsString = implode('|', $websiteTaxPostIDs);

            return $websiteTaxPostIDsString;
            
        	
        }


    }

}
