<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Products_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with Customer Roles and User Info.
     *
     * @since 1.0.0
     */
    class MM_REST_Products_V1_Controller extends WC_REST_Controller {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of MM_REST_Customer_V1_Controller.
         *
         * @var MM_REST_Products_V1_Controller
         */
        private static $_instance;


        /**
         * Endpoint namespace.
         *
         * @var string
         */
        protected $namespace = 'mm/v1';

        /**
         * Route base.
         *
         * @var string
         */
        protected $rest_base = 'products';
        
        
        //Other properties
        private $debug_active = null;


        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_Customer_V1_Controller constructor.
         *
         * @since 1.0.0
         * @access public
         */
        public function __construct() {



            
            // Fires when preparing to serve an API request.
            add_action("rest_api_init", array($this, "register_routes"));
            
            //Fires when the WC API is called to add/update a product or variation
            add_action("woocommerce_rest_insert_product_object", array($this, "create_update_product"), 10, 3);
            add_action("woocommerce_rest_insert_product_variation_object", array($this, "create_update_product"), 10, 3);
        }
        /**
         * Register routes for ManageMore roles API.
         *
         * @since 1.0.0
         * @access public
         */
        public function register_routes() {


            register_rest_route($this->namespace, '/' . $this->rest_base, array(
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'update_items'),
                    'permission_callback' => array($this, 'update_item_permissions_check'),
                ),
            ));
        }


        /**
         * Check if a given request has access to update an item.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function update_item_permissions_check($request) {

            $id = $request[0]['id'];
            if (!wc_rest_check_post_permissions('product', 'edit', $id)) {
                return new WP_Error('mm_rest_cannot_edit', __('Sorry, you are not allowed to edit this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }
          

        /**
         * Fires when a product/variation is created or updated via standard WC rest.
         * Allows us to pass our customer_pricing field to the standard product create/update
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function create_update_product($product, $request, $create_product){
        	
       	     $product_id = $product->get_id();

             //If customer pricing passed, update it
             if (isset($request['customer_pricing'])){
             	 $customer_pricing = $request['customer_pricing'];  //should be a semicolon/pipe delimited string of price rules ("id;price;method;startdate;enddate|id;price;method;startdate;enddate")
             	 $this->debug_log('Customer Pricing update for Product Id:'.$product_id.' ['.$customer_pricing.']');
             	 MM_Connector_Functions::update_customer_pricing($customer_pricing,$product_id,2);  
             }

        	   return;
        }

        /**
         * Update WooCommerce Items.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
         public function update_items($request) {
             $products_data = $request->get_json_params();
             $update_results = [];
             $product_count = 0 ;
             $update_count = 0 ;
             $ignore_count = 0 ;
             $error_count = 0 ;
             

             if (is_array($products_data)) {
                 foreach ($products_data as $product_data) {
                     $product_id = isset($product_data['id']) ? intval($product_data['id']) : 0;
                     $product = wc_get_product($product_id);
                     $updated = false; // Flag to track if any update is made
                     $product_count ++ ;
                     if (!$product) {
                         $update_results[] = ['id' => $product_id, 'status' => 'Error: ID not found'];
                         $this->debug_log('Error: ID not found ['. $product_id . '] when syncing quantity for SKU: ' . $product_data['sku']);
                         $error_count ++ ;
                         continue;
                     }

                     // Validate SKU
                 	   $product_sku = $product_data['sku'];
                     if (isset($product_data['sku']) && $product_data['sku'] !== $product->get_sku()) {
                         $update_results[] = ['id' => $product_id, 'status' => 'Error: Mismatched SKU'];
                         $this->debug_log('Error: Mismatched SKU when syncing quantity for SKU: ' . $product_data['sku']);
                         $error_count ++ ;
                         continue;
                     }

                     // Update product status if it's different
                     if (isset($product_data['status']) && in_array($product_data['status'], ['draft', 'private', 'publish']) && $product_data['status'] !== $product->get_status()) {
                         $product->set_status($product_data['status']);
                         $updated = true;
                     }

                     // Update catalog visibility if it's different
                     if (isset($product_data['catalog_visibility']) && in_array($product_data['catalog_visibility'], ['visible', 'catalog', 'search', 'hidden']) && $product_data['catalog_visibility'] !== $product->get_catalog_visibility()) {
                     	
                     	   $update_details = 'catalog_visibility - MM Value: ' . $product_data['catalog_visibility'] . ' Woo Value: ' . $product->get_catalog_visibility();
                     	   $this->debug_log('sku: ' . $product_sku . ' id: ' . $product_id .' Updated.' . $update_details);
                         $product->set_catalog_visibility($product_data['catalog_visibility']);
                         $updated = true;
                     }

                     // Handle stock and other properties if they're different
                     if (isset($product_data['manage_stock'])) {
                         $manage_stock = $product_data['manage_stock'] === true;
                         if ($manage_stock !== $product->get_manage_stock()) {
                        	   $update_details = 'manage_stock - MM Value: ' . $product_data['manage_stock'] . ' Woo Value: ' . $product->get_manage_stock();
                        	   $this->debug_log('sku: ' . $product_sku . ' id: ' . $product_id .' Updated.' . $update_details);
                             $product->set_manage_stock($manage_stock);
                             $updated = true;
                         }
                     }

                     if ($product->get_manage_stock() && isset($product_data['stock_quantity']) && intval($product_data['stock_quantity']) !== $product->get_stock_quantity()) {
                     	   $update_details = 'stock_quantity - MM Value: ' . $product_data['stock_quantity'] . ' Woo Value: ' . $product->get_stock_quantity();
                    	   $this->debug_log('sku: ' . $product_sku . ' id: ' . $product_id .' Updated.' . $update_details);
                         $product->set_stock_quantity($product_data['stock_quantity']);
                         $updated = true;
                     }

                     if (isset($product_data['stock_status'])) {
                         $in_stock_status = $product_data['stock_status'];
                         if ($in_stock_status !== $product->get_stock_status()) {
                        	   $update_details = 'stock_status - MM Value: ' . $product_data['stock_status'] . ' Woo Value: ' . $product->get_stock_status();
                        	   $this->debug_log('sku: ' . $product_sku . ' id: ' . $product_id .' Updated.' . $update_details);
                             $product->set_stock_status($in_stock_status);
                             $updated = true;
                         }
                     }

                     if ($updated) {
                         $product->save();
                         $update_results[] = ['id' => $product_id, 'status' => 'Updated'];
                         $update_count ++ ;
                     } else {
                         $update_results[] = ['id' => $product_id, 'status' => 'Unchanged: No update required'];
                         $ignore_count ++ ;
                     }
                 }
             }

             $this->debug_log('Quantity Sync: ' . $product_count . ' products; ' . $update_count . ' updated; '.$ignore_count . ' no update required; ' . $error_count . ' errors.');

             return new WP_REST_Response($update_results, 200);
         }

        protected function debug_log($message) {
        	  if ($this->debug_active === null){
        	  	$this->debug_active = MM_Debug_Mode::is_active(MM_Debug_Mode::REST_API);
        	  }
            if ($this->debug_active) {
               MM_Connector_Functions::debug_log($message, MM_Debug_Mode::REST_API);
            }
        }


    }  //end of class 
    
    
   
}
