<?php 

if (!defined('ABSPATH')) exit; // Exit if accessed directly

/**
 * Model that houses the logic of MM WooCommerce Barcode Reader Shortcode
 *
 * @since 5.0.1
 */

if (!class_exists('MM_WOO_BARCODE_SCANNER')) {

    class MM_WOO_BARCODE_SCANNER {
        
        public $mm_custom_upc_field = 'mm_product_upc'; // Custom UPC field (deprecated, but older mm versions may still write to this field))
        public $mm_retail_upc_field = 'mm_retail_upc'; // Custom Retail UPC field
        private $auto_add = false;
        private $success_text;
        private $not_found_text;
        private $default_text = 'Scan to Search';
        private $default_auto_add_text = 'Scan to Add';
        private $close_text = 'Close Scanner';
        private $default_lookup_text = 'Product found - %product_name% [%barcode%]';
        private $default_add_text = '1 - %product_name% added to cart.';
        private $default_not_found_text = 'No product found for barcode: %barcode%.';

        public function __construct() {
            add_shortcode('mm_barcode_scanner', array($this, 'render_barcode_scanner'));
            add_action('wp_ajax_mm_scan_barcode', array($this, 'handle_barcode_scan'));
            add_action('wp_ajax_nopriv_mm_scan_barcode', array($this, 'handle_barcode_scan'));
        }

        public function enqueue_barcode_scripts() {
            wp_enqueue_script('quagga', 'https://cdnjs.cloudflare.com/ajax/libs/quagga/0.12.1/quagga.min.js', array(), null, true);
            wp_enqueue_script('mm-barcode-scanner', MM_JS_URL . 'mm-connector-barcode-scanner.js', array('jquery'), null, true);

            wp_enqueue_style('mm-barcode-scanner-css', MM_CSS_URL . 'mm-connector-barcode-scanner.css', array(), null);

            // Pass auto_add value to JavaScript
            wp_localize_script('mm-barcode-scanner', 'mmBarcodeScanner', array(
               'auto_add' => $this->auto_add,
               'success_text' => $this->success_text,
               'not_found_text' => $this->not_found_text,
               'ajax_url' => admin_url('admin-ajax.php'),
            ));

        }

        public function render_barcode_scanner($atts) {
        	

             // Extract shortcode attributes with defaults
             $atts = shortcode_atts([
                 'success_text' => '',
                 'not_found_text' => $this->default_not_found_text,
                 'button_text' => $this->default_text,
                 'close_text' => $this->close_text,
                 'auto_add' => 'false',
             ], $atts, 'mm_barcode_scanner');

             // Store shortcode parameters as class properties
             $this->success_text = $atts['success_text'];
             $this->not_found_text = $atts['not_found_text'];
             $this->close_text = $atts['close_text'];

             // Convert `auto_add` to a lower value
             $this->auto_add = strtolower($atts['auto_add']);

             // Set button text
             $button_text = esc_html($atts['button_text']);
             
             // If auto_add is enabled and button text is the default, switch it
             if (($this->auto_add === 'true' || $this->auto_add === 'cart') && $button_text === $this->default_text) {
                 $button_text = $this->default_auto_add_text;
             }
   

            // Ensure scripts are enqueued in the footer
            add_action('wp_footer', array($this, 'enqueue_barcode_scripts'));

            ob_start();

            ?>
            <div id="scanner-button-container" style="display:block; position: relative; max-width: 400px; width: 100%;">
                <button id="mm-scan-barcode" class="button" style="width:100%; margin-top:10px;"><?php echo $button_text; ?></button>
            </div>
            <div id="scanner-container" style="display:none; position: relative; max-width: 400px; width: 100%;">
                <video id="barcode-scanner" autoplay playsinline style="width: 100%; background: black;"></video>
                <div id="laser-line"></div>
                <button id="close-scanner" class="button" style="width:100%; margin-top:10px;"><?php echo $this->close_text; ?></button>
            </div>

            <?php
            return ob_get_clean();

        }


        public function handle_barcode_scan() {
            if (!isset($_POST['barcode'])) {
                wp_send_json_error(array('message' => 'No barcode provided.'));
            }

            $barcode = sanitize_text_field($_POST['barcode']);
            $product_data = $this->find_product_by_barcode($barcode);
            
            $auto_add = isset($_POST['auto_add']) && ($_POST['auto_add'] === 'true' || $_POST['auto_add'] ==='cart');
            $this->success_text = $_POST['success_text'];
            $this->not_found_text = $_POST['not_found_text'];

            if ($product_data) {
            
                $product_url = get_permalink($product_data['parent_id']); // Always parent page
                $product_id_to_add = $product_data['variation_id'];    // Variation Id or Regular Product Id
                $variable_product = $product_data['variable_product']; // Is it a Variation?
                $product = wc_get_product($product_id_to_add);
                $product_name = $product->get_name();
                $quantity = 1;
                $valid = apply_filters('woocommerce_add_to_cart_validation', true, $product_id_to_add, $quantity);

                if ($auto_add) {
                	  if (!$valid){
                        // Return product page URL, and variable flag
                        $not_added_text = 'Product could not be added to the cart';
                        //Notice not necessary, because woocommerce_add_to_cart_validation would have posted one
                        wp_send_json_error(['message' => $not_added_text]);
                        return;
                	  	
                	  }
                    $cart_item_key = WC()->cart->add_to_cart($product_id_to_add);
                    if ($cart_item_key) {
                        $success_text_add = !empty($this->success_text) ? $this->success_text : $this->default_add_text;
                        $success_text_add = str_replace(['%barcode%', '%product_name%'], [$barcode, $product_name], $success_text_add);
                        wc_add_notice($success_text_add, 'success');
                    } else {
                        error_log("Failed to add product $product_id_to_add to cart.");
                    }
                } else {
                    $success_text_lookup = !empty($this->success_text) ? $this->success_text : $this->default_lookup_text;
                    $success_text_lookup = str_replace(['%barcode%', '%product_name%'], [$barcode, $product_name], $success_text_lookup);
                    wc_add_notice($success_text_lookup, 'success');
                }

                // Return product page URL, and variable flag
                wp_send_json_success(array('product_url' => $product_url,
                                           'variable_product' => $variable_product,
                                          ));
            } else {
                $not_found_text = str_replace('%barcode%', $barcode, $this->not_found_text);
                wc_add_notice($not_found_text, 'error');
                wp_send_json_error(['message' => $not_found_text]);
            }
        }


        private function find_product_by_barcode($barcode) {

            // Attempt to find product by SKU
            $product_id = wc_get_product_id_by_sku($barcode);
            
            //Check by Woo Standard Barcode field
            if (!$product_id) {
               $product_id = wc_get_product_id_by_global_unique_id($barcode);
            }
            
            //Check by our alias field and custom upc field (now deprecated, but older mm versions may still write to this field)
            if (!$product_id) {

               global $wpdb;
               
               $query = $wpdb->prepare("
                   SELECT post_id
                   FROM {$wpdb->postmeta}
                   WHERE meta_key IN (%s, %s)
                   AND meta_value = %s
                   LIMIT 1
               ", $this->mm_custom_upc_field, $this->mm_retail_upc_field, $barcode);

               $product_id = $wpdb->get_var($query);

            }

            if (!$product_id) {
            	  //Not Found
                return false;
            }
 
            $product = wc_get_product($product_id);

            // If it's a variation, get the parent product
            if ($product->is_type('variation')) {
                $parent_id = $product->get_parent_id();
                return [
                    'parent_id' => $parent_id,
                    'variation_id' => $product_id,
                    'variable_product' => true,
                ];
            }

            // For simple products, return the same id for both
            return [
                'parent_id' => $product_id,
                'variation_id' => $product_id,
                'variable_product' => false,
            ];

        }

    }
    
    new MM_WOO_BARCODE_SCANNER();
}

