<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_Connector_Functions')) {

    /**
     * Model that houses plugin helper functions.
     *
     * @since 1.0.0
     */
    final class MM_Connector_Functions {


        private static $logger;
        private static $log_context;
        private static $debug_mode;

        /**
         * Utility function that determines if a plugin is active or not.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $plugin_basename Plugin base name. Ex. woocommerce/woocommerce.php
         * @return boolean True if active, false otherwise.
         */
        public static function is_plugin_active($plugin_basename) {

            // Makes sure the plugin is defined before trying to use it
            if (!function_exists('is_plugin_active')) {
                include_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            return is_plugin_active($plugin_basename);

        }
        
        /**
         * Select List Options for a Post Type.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $post_type Post Type for list
         * @param string $selected_value Selected item in list
         * @return string HTML of selection drop list
         */
        public static function post_type_selection_options( $post_type, $selected_value = '', $allow_blank = false){
            $posts = get_posts(
                array(
                    'post_type'  => $post_type,
                    'post_status' => 'publish',
                    'numberposts' => -1
                )
            );
            if( ! $posts ) return;
            if ($allow_blank == true){
            	 if ($selected_value == ''){
            	 	  $options_html = '<option value="" selected="selected">&nbsp;</option>';
            	 }
            	 else {
            	 	  $options_html = '<option value="" ></option>';
            	  	
            	 }
            }

            foreach( $posts as $p )
            {
            	  $post_id = $p->ID;
                if( $post_id == $selected_value)  {
                   $options_html .= '<option value="' . $post_id . '" selected="selected">' . esc_html( $p->post_title ) . '</option>';
                }
                else {
                   $options_html .= '<option value="' . $post_id . '">' . esc_html( $p->post_title ) . '</option>';
                }
            }
            return $options_html;
        }

       /**
         * Udpate Customer Specific Pricing for a particular user or product.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $customer_pricing Semicolon/pipe delimited string of price rules ("id;amount;method;startdate;enddate|id;price;amount;startdate;enddate")
         * @param integer $pricing_id The id of the user/product we are updating (id in price rule is the opposite value)
         * @param integer $pricing_type Type of pricing we are updating (1=User, 2=Product)
         * @return string HTML of selection drop list
         */
        public static function update_customer_pricing($customer_pricing,$pricing_id,$pricing_type){
      	
        	  global $wpdb;
        	  
           //Check if Customer Pricing Enabled
           $customer_pricing_enable = get_option('mm_customer_pricing_enable',1);
           if($customer_pricing_enable != 1){
           	 return; //do nothing
           }
 
           //First explode the string by pipe
           $pricing_sets = explode('|',$customer_pricing);
           

           //Next, delete current rules
           
           //Old Code - depricated 2.0.0
/*           $meta_key = ($pricing_type == 1) ? 'user_id' : 'product_id';
           $posts = get_posts(
               array(
                   'post_type'  => 'mm-customer-price',
                   'post_status' => 'publish',
                   'meta_key' => $meta_key,
                   'meta_value' => $pricing_id,
                   'numberposts' => -1
               )
           );
           if ($posts){
           	  foreach ($posts as $post) {
           	    wp_delete_post($post->ID, true ); //skip trash
           	  } 
           }
*/

          //2.0.0 - Use new database table for efficiency.
          $table_name =  $wpdb->prefix . 'mm_customer_price';
          $key_field = ($pricing_type == 1) ? 'user_id' : 'product_id';
          $wpdb->delete( $table_name, array( $key_field => $pricing_id ) );


           //1.1.3 - Moved this code here, after deletion.  If blank string is passed, this means to delete all rules.
           if (empty($pricing_sets)){
           	 return;
           }

           
           //1.0.4 - Added deferring these to speed up addition of posts
           //2.0.0 - Removed; We no longer use posts here
           //wp_defer_term_counting( true );
           //wp_defer_comment_counting( true );

           //Finally, for each pricing set (id;amount;method;startdate;enddate), explode the set by semicolon and add the rule (id is dependent on pricing_type)
           foreach ($pricing_sets as $pricing_set) {

               $pricing_rule = explode(';',$pricing_set);  
               $user_id = ($pricing_type == 1) ? $pricing_id : $pricing_rule[0] ; 
               $product_id =($pricing_type != 1) ? $pricing_id : $pricing_rule[0] ;
               $amount = $pricing_rule[1];
               $method = $pricing_rule[2];
               $start_date = $pricing_rule[3];
               $end_date = $pricing_rule[4];
               
               if ($user_id == '' || $product_id == ''){
               	continue; // Cannot create rules with No Id.
               }


               //Add the rule as a mm-customer-price post type

               //Old Code - depricated 2.0.0
/*	            $customer_price = array(
                  'post_title'    => '',
                  'post_content'  => '',
                  'post_type'     => 'mm-customer-price',
                  'post_status'   => 'publish',
                  'meta_input' => array(
                                 'user_id'=>$user_id,
                                 'product_id'=>$product_id,
                                 'amount'=>$amount,
                                 'method'=>$method,
                                 'start_date'=>$start_date,
                                 'end_date'=>$end_date,
                                 ),
                  );
               $customer_price_id = wp_insert_post( $customer_price );
*/               
               //2.0.0 - Use database file for efficiency
               $wpdb->insert( 
                           $table_name, 
                           array( 
	                             'user_id' => $user_id, 
	                             'product_id' => $product_id, 
	                             'method' => $method, 
	                             'amount' => $amount, 
	                             'start_date' => $start_date, 
	                             'end_date' => $end_date 
                                ),
                            array('%d','%d','%d','%f','%s','%s')
                            );


           }  
           
           //1.0.4 - Rearm items deferred these to speed up addition of posts
           //2.0.0 - Removed; We no longer use posts here
           //wp_defer_term_counting( false );
           //wp_defer_comment_counting( false );
          
        	  return;
        	  
        	   
        }
    
       /**
         * Retrieve a plug in setting, and provide a default. Used for text settings so that defaults are set here even if setting was not saved.
         * This avoids having to provide this default wherever get_option is used to retrieve this setting.
         *
         * @since 3.0.4
         * @access public
         *
       */
        public static function mm_get_option($option_name, $default = '', $return_default = false ) {
        
        
        $defaults = array(
            'mm_disabled_account_message'=>'This account has been temporarily disabled.',
            'mm_inactive_account_message'=>'This account has been deactivated.',
            'mm_license_expired_message'=> '{notation} One or more items require a valid {license_name}. <a href="/account/license">Click here to upload a new one.</a>',
            'mm_customer_license_1_name' => 'License 1',
            'mm_customer_license_2_name' => 'License 2',
            'mm_customer_license_3_name' => 'License 3',
            'mm_payment_checkout_message' => 'Thank you for your payment. Your online balance will not reflect this payment immediately. Please allow 24-48 hours for your balance to be updated online.',
            'mm_indirect_tax_warning_message' => '&#x1F4CC; The price shown on the denoted cart items below includes applicable excise taxes for your state.',
            'mm_indirect_tax_detail_message' => '&#x1F4CC;',  //Push Pin  --- Could also use something like '<br>(Excise Tax: {indirect_tax_amount})'
            'mm_license_expired_notation_1' =>  '&#x1F4E2;',  //PA Loudspeaker
            'mm_license_expired_notation_2' =>  '&#x1F4E3;',  //Megaphone
            'mm_license_expired_notation_3' =>  '&#x1F514;',  //Bell
            'mm_region_restricted_notation' =>  '&#128205;',  //Map PIN
            'mm_region_restricted_message'  =>  '&#128205; One or more items cannot be shipped to the location provided.',
            // ...add other default values here...
        );

        if ($return_default) {
        	  return $defaults[$option_name] ;
        }

        // If there's a default value specified in the function call, use it.
        // Otherwise, see if there's a default value specified in the defaults array.
        $default_value = isset($default) ? $default : $defaults[$option_name] ;

        return get_option($option_name, $default_value);
       }

       public static function get_revita_url($test_mode = 'no'){
           //Check if card processing should be performed in sandbox mode
           if ($test_mode!='yes'){
              $url = 'https://api.authorize.net/xml/v1/request.api';
           }
           else {
        	    $url = 'https://apitest.authorize.net/xml/v1/request.api';
           }
           return $url;
       	  
       }

       public static function get_payment_gateway_setting($gateway_id, $setting_key) {
          $options = get_option('woocommerce_' . $gateway_id . '_settings');
          if (is_array($options) && isset($options[$setting_key])) {
             return $options[$setting_key];
          }
          return null; // Not found
       }       
       
       // Method to initialize logging related properties
       public static function init_logging() {
           self::$debug_mode = get_option('mm_debug_mode', 0);
           self::$logger = wc_get_logger();
           self::$log_context = array('source' => 'managemore-connector');
       }

       public static function debug_log($message,$debug_mode = 0) {

           // Ensure logging is initialized
           if (null === self::$logger || null === self::$debug_mode) {
               self::init_logging();
           }

           //if we are in the selected debug mode or all, log the event
           if ($debug_mode == self::$debug_mode || self::$debug_mode == 99 ) {
               self::$logger->log('debug', $message, self::$log_context);
           }
           return;
       }
    
    
    }
}
 