<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_Connector_Functions')) {

    /**
     * Model that houses plugin helper functions.
     *
     * @since 1.0.0
     */
    final class MM_Connector_Functions {


        private static $logger;
        private static $log_context;
        private static $debug_mode;
        // Static properties to cache relevant settings
        protected static $customerPricingEnable = null;
        protected static $mm_allow_cache = null;
        

        /**
         * Utility function that determines if a plugin is active or not.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $plugin_basename Plugin base name. Ex. woocommerce/woocommerce.php
         * @return boolean True if active, false otherwise.
         */
        public static function is_plugin_active($plugin_basename) {

            // Makes sure the plugin is defined before trying to use it
            if (!function_exists('is_plugin_active')) {
                include_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            return is_plugin_active($plugin_basename);

        }
        
        /**
         * Select List Options for a Post Type.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $post_type Post Type for list
         * @param string $selected_value Selected item in list
         * @return string HTML of selection drop list
         */
        public static function post_type_selection_options( $post_type, $selected_value = '', $allow_blank = false){
            $posts = get_posts(
                array(
                    'post_type'  => $post_type,
                    'post_status' => 'publish',
                    'numberposts' => -1
                )
            );
            if( ! $posts ) return;
            if ($allow_blank == true){
            	 if ($selected_value == ''){
            	 	  $options_html = '<option value="" selected="selected">&nbsp;</option>';
            	 }
            	 else {
            	 	  $options_html = '<option value="" ></option>';
            	  	
            	 }
            }

            foreach( $posts as $p )
            {
            	  $post_id = $p->ID;
                if( $post_id == $selected_value)  {
                   $options_html .= '<option value="' . $post_id . '" selected="selected">' . esc_html( $p->post_title ) . '</option>';
                }
                else {
                   $options_html .= '<option value="' . $post_id . '">' . esc_html( $p->post_title ) . '</option>';
                }
            }
            return $options_html;
        }

       /**
         * Udpate Customer Specific Pricing for a particular user or product.
         *
         * @since 1.0.0
         * @access public
         *
         * @param string $customer_pricing Semicolon/pipe delimited string of price rules ("id;amount;method;startdate;enddate|id;price;amount;startdate;enddate")
         * @param integer $pricing_id The id of the user/product we are updating (id in price rule is the opposite value)
         * @param integer $pricing_type Type of pricing we are updating (1=User, 2=Product)
         * @return string HTML of selection drop list
         */
        public static function update_customer_pricing($customer_pricing,$pricing_id,$pricing_type){
      	
        	 global $wpdb;
        	  
           //Check if Customer Pricing Enabled
           if (self::$customerPricingEnable === null) {
               self::$customerPricingEnable = get_option('mm_customer_pricing_enable', 1);
           }

           if (self::$customerPricingEnable != 1) {
               return; // Do nothing if pricing is disabled
           }

           //Check if Cache Enabled
           if (self::$mm_allow_cache === null) {
               self::$mm_allow_cache = get_option('mm_allow_cache', 0);
           }
 

           // Establish the table name and key field based on pricing type
           $table_name = $wpdb->prefix . 'mm_customer_price';
           $key_field = $pricing_type == 1 ? 'user_id' : 'product_id';


           // Start a database transaction
           $wpdb->query('START TRANSACTION');

           // Delete existing pricing rules
           $wpdb->delete($table_name, [$key_field => $pricing_id]);

           // If the input is empty, no need to proceed further
           if (empty($customer_pricing)){
               $wpdb->query('COMMIT'); // Commit the transaction
               if (self::$mm_allow_cache){
               	 self::remove_pricing_cache($pricing_id, $pricing_type);
               }
               return;
           }


           // Prepare for bulk insertion
           $values = [];
           $pricing_sets = explode('|', $customer_pricing);
           foreach ($pricing_sets as $pricing_set) {
               $pricing_rule = explode(';', $pricing_set);
               $user_id = $pricing_type == 1 ? $pricing_id : $pricing_rule[0];
               $product_id = $pricing_type != 1 ? $pricing_id : $pricing_rule[0];

               if (empty($user_id) || empty($product_id)) {
                   continue; // Skip invalid entries
               }

               // Prepare the data for bulk insertion
               $values[] = $wpdb->prepare("(%d, %d, %s, %f, %s, %s)", 
                                          $user_id, $product_id, $pricing_rule[2], 
                                          $pricing_rule[1], $pricing_rule[3], $pricing_rule[4]);
                
           }

           // Perform bulk insert if there are values to insert
           if (!empty($values)) {
               $query = "INSERT INTO $table_name (user_id, product_id, method, amount, start_date, end_date) VALUES ";
               $query .= implode(', ', $values);
               $wpdb->query($query);
           }

           $wpdb->query('COMMIT'); // Commit the transaction

           if (self::$mm_allow_cache){
             	 self::remove_pricing_cache( $pricing_id, $pricing_type);
           }

           return;
        	   
        }


        private static function remove_pricing_cache($pricing_id, $pricing_type){

            if ($pricing_type == 2) {
               MM_Object_Cache::delete_by_scope('product', $pricing_id, 'cust_price');
            } else {
               MM_Object_Cache::delete_by_scope('user', $pricing_id, 'cust_price');
            }

        	  /*
            foreach (array_keys($affected_products) as $product_id) {
               if ($pricing_type == 2) {
                  MM_Customer_Pricing_Cache::delete_all_for_product($product_id);
               } else {
                  MM_Customer_Pricing_Cache::delete($product_id, $pricing_id); // pricing_id = user_id here
               }
            }       	
            */

        }

    
       /**
         * Retrieve a plug in setting, and provide a default. Used for text settings so that defaults are set here even if setting was not saved.
         * This avoids having to provide this default wherever get_option is used to retrieve this setting.
         *
         * @since 3.0.4
         * @access public
         *
       */
        public static function mm_get_option($option_name, $default = '', $return_default = false ) {
        
        
        $defaults = array(
            'mm_disabled_account_message' => 'This account has been temporarily disabled.',
            'mm_inactive_account_message' => 'This account has been deactivated.',
            'mm_login_required_message' => 'You must be logged in to purchase this product.',
            'mm_license_expired_message' => '{notation} This item requires a valid {license_name}.',
            'mm_payment_checkout_message' => 'Thank you for your payment. Your online balance will not reflect this payment immediately. Please allow 24-48 hours for your balance to be updated online.',
            'mm_indirect_tax_warning_message' => '{notation} The price shown on the denoted cart items includes applicable excise taxes for your location.',
            'mm_indirect_tax_note_message' => 'Price includes excise tax',
            'mm_indirect_tax_detail_notation' => '&#x1F4CC;',
            'mm_license_expired_notation_1' => '&#x1F4E2;',
            'mm_license_expired_notation_2' => '&#x1F4E3;',
            'mm_license_expired_notation_3' => '&#x1F514;',
            'mm_region_restricted_notation' => '&#128205;',
            'mm_region_restricted_message' => '{notation} This item cannot be shipped to the location provided.',
            // ...add other default values here...
        );

        if ($return_default) {
        	  return $defaults[$option_name] ;
        }

        // If there's a default value specified in the function call, use it.
        // Otherwise, see if there's a default value specified in the defaults array.
        if($default !== ''){
           return get_option($option_name, $default);
        }
        elseif(isset($defaults[$option_name])){
        	 return get_option($option_name,$defaults[$option_name]);
        }
        else {
           return get_option($option_name);
        }
        
        
        
       }

       public static function get_revita_url($test_mode = 'no'){
           //Check if card processing should be performed in sandbox mode
           if ($test_mode!='yes'){
              $url = 'https://api.authorize.net/xml/v1/request.api';
           }
           else {
        	    $url = 'https://apitest.authorize.net/xml/v1/request.api';
           }
           return $url;
       	  
       }

       public static function get_payment_gateway_setting($gateway_id, $setting_key) {
          $options = get_option('woocommerce_' . $gateway_id . '_settings');
          if (is_array($options) && isset($options[$setting_key])) {
             return $options[$setting_key];
          }
          return null; // Not found
       }       
       
       // Method to initialize logging related properties
       public static function init_logging() {
           self::$debug_mode = get_option('mm_debug_mode', 0);
           self::$logger = wc_get_logger();
           self::$log_context = array('source' => 'managemore-connector');
       }

       public static function debug_log($message,$debug_mode = 0) {

           // Ensure logging is initialized
           if (null === self::$logger || null === self::$debug_mode) {
               self::init_logging();
           }

           //if we are in the selected debug mode or all, log the event
           if ($debug_mode == self::$debug_mode || self::$debug_mode == 99 ) {
               self::$logger->log('debug', $message, self::$log_context);
           }
           return;
       }
    
    
    }
}
 