<?php 

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

//Require List Table Class we are extending
if ( ! class_exists( 'WP_List_Table' ) ) {
   require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class MM_Indirect_Tax_List extends WP_List_Table {

   /** Class constructor */
   public function __construct() {

   parent::__construct( [
      'singular' => __( 'Indirect Tax Rule', 'managemore-connector' ), //singular name of the listed records
      'plural' => __( 'Indirect Tax Rules', 'managemore-connector' ), //plural name of the listed records
      'ajax' => false //should this table support ajax?

      ] );

   }

   /**
   * Retrieve indirect tax data from the database
   *
   * @param int $per_page
   * @param int $page_number
   *
   * @return mixed
   */
   public static function get_indirect_taxes( $per_page = 50, $page_number = 1 ) {

      global $wpdb;

      $sql = "SELECT * FROM {$wpdb->prefix}mm_indirect_tax";

      if ( ! empty( $_REQUEST['orderby'] ) ) {
         $sql .= ' ORDER BY ' . esc_sql( $_REQUEST['orderby'] );
         $sql .= ! empty( $_REQUEST['order'] ) ? ' ' . esc_sql( $_REQUEST['order'] ) : ' ASC';
      }

      $sql .= " LIMIT $per_page";

      $sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;

      $result = $wpdb->get_results( $sql, 'ARRAY_A' );

      return $result;

   }

   /**
   * Delete a indirect tax record.
   *
   * @param int $id customer price ID
   */
   public static function delete_indirect_tax( $id ) {
      global $wpdb;

      $wpdb->delete(
         "{$wpdb->prefix}mm_indirect_tax",
         [ 'id' => $id ],
         [ '%d' ]
         );
      }

   /**
   * Delete all indirect tax records.
   *
   *
   */
   public static function delete_all_indirect_taxes() {
      global $wpdb;
      $delete = $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}mm_indirect_tax");
   }


   /**
   * Returns the count of records in the database.
   *
   * @return null|string
   */
   public static function record_count() {
      global $wpdb;

      $sql = "SELECT COUNT(*) FROM {$wpdb->prefix}mm_indirect_tax";

      return $wpdb->get_var( $sql );
   }

   /** Text displayed when no indirect tax data is available */
   public function no_items() {
      _e( 'No indirect tax rates avaliable.', 'sp' );
   }

   /**
   * Render a column when no column specific method exists.
   *
   * @param array $item
   * @param string $column_name
   *
   * @return mixed
   */
   public function column_default( $item, $column_name ) {

   	global $woocommerce;
   	
      switch ( $column_name ) {
      case 'indirect_tax_type':
         $indirect_tax_type_id = $item[ $column_name ];
         $indirect_tax_name = get_the_title($indirect_tax_type_id);  //Type name is in its own post type as the Title
         return $indirect_tax_name;
      case 'indirect_tax_state':
         return $item[ $column_name ];      
      case 'indirect_tax_calc_method':
         $method_description = array('','Percent of Cost','Flat Rate','Net Weight','Unit 1 Basis','Unit 2 Basis',
                                        'Unit 3 Basis','','','','Percent of Price'); //note mm values are 1-based and there is no 7-9
         $method = $item[ $column_name ];
         return $method_description[$method];
      case 'indirect_tax_rate':
         //2.1.5 - Changed to show the correct number of decimals in this field
         $amount = number_format($item[ $column_name ],4);     
         return $amount;
      
      default:
         return print_r( $item, true ); //Show the whole array for troubleshooting purposes
      }
   }

   /**
   * Render the bulk edit checkbox
   *
   * @param array $item
   *
   * @return string
   */
   function column_cb( $item ) {
      return sprintf(
      '<input type="checkbox" name="bulk-delete[]" value="%s" />', $item['id']
      );
   }

   /**
   * Associative array of columns
   *
   * @return array
   */
   function get_columns() {
      $columns = [
      'cb' => '<input type="checkbox" />',
      'indirect_tax_type' => __( 'Indirect Tax Type', 'managemore-connector' ),
      'indirect_tax_state' => __( 'State', 'managemore-connector' ),
      'indirect_tax_calc_method' => __( 'Method', 'managemore-connector' ),
      'indirect_tax_rate' => __( 'Rate', 'managemore-connector' )
      ];

      return $columns;
   }

   /**
   * Columns to make sortable.
   *
   * @return array
   */
   public function get_sortable_columns() {
      $sortable_columns = array(
      'indirect_tax_type' => array( 'indirect_tax_type', true ),
      'indirect_tax_state' => array( 'indirect_tax_state', false )
      );

      return $sortable_columns;
   }

   /**
   * Returns an associative array containing the bulk action
   *
   * @return array
   */
   public function get_bulk_actions() {
      $actions = [
      'bulk-delete' => 'Delete',
      'delete-all' => 'Delete All'
      ];

      return $actions;
   }   

   /**
   * Handles data query and filter, sorting, and pagination.
   */
   public function prepare_items() {

      $this->_column_headers = $this->get_column_info();

      /** Process bulk action */
      $this->process_bulk_action();

      $per_page = $this->get_items_per_page( 'indirect_taxes_per_page', 50 );
      $current_page = $this->get_pagenum();
      $total_items = self::record_count();

      $this->set_pagination_args( [
      'total_items' => $total_items, //WE have to calculate the total number of items
      'per_page' => $per_page //WE have to determine how many items to show on a page
      ] );

      $this->items = self::get_indirect_taxes( $per_page, $current_page );
   }



   /**
   * Process the Bulk Action
   */
   public function process_bulk_action() {

      //Detect when a single delete action is being triggered... (not currently implemented, would require custom column function)
/*      if ( 'delete' === $this->current_action() ) {

         // In our file that handles the request, verify the nonce.
         $nonce = esc_attr( $_REQUEST['_wpnonce'] );

         if ( ! wp_verify_nonce( $nonce, 'mm_delete_indirect_tax' ) ) {
            die( 'Invalid Request' );
         }
         else {
            self::delete_indirect_tax( absint( $_GET['indirect_tax'] ) );

            wp_redirect( esc_url( add_query_arg() ) );
            exit;
         }

      }
*/

      // If the delete bulk action is triggered (delete selected items)
      if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-delete' )
      || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-delete' )
      ) {

         $delete_ids = esc_sql( $_POST['bulk-delete'] );

         // loop over the array of record IDs and delete them
         foreach ( $delete_ids as $id ) {
         self::delete_indirect_tax( $id );

         }

         wp_redirect( esc_url( add_query_arg() ) );
         exit;
      }
      
      // If the delete all bulk action is triggered (delete all items)
      if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'delete-all' )
      || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'delete-all' )
      ) {

         self::delete_all_indirect_taxes( );


         wp_redirect( esc_url( add_query_arg() ) );
         exit;
      }
      
      
      
   }

} 