<?php 

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if (!class_exists('MM_WOO_ORDER_STATUS')) {

    /**
     * Model that houses the logic of MM WooCommerce Order Status Related Functions (Ready for Pickup)
     *
     * @since 5.0.1
     */
    class MM_WOO_ORDER_STATUS {

       /**
        * Registers the "Ready for Pickup" custom order status.
        */
       public function mm_rfp_register_custom_order_status() {
           register_post_status('wc-pickup-ready', array(
               'label'                     => _x('Ready for Pickup', 'WooCommerce Order Status', 'woocommerce'),
               'public'                    => true,
               'show_in_admin_status_list'  => true,
               'show_in_admin_all_list'     => true,
               'label_count'                => _n_noop('Ready for Pickup <span class="count">(%s)</span>', 'Ready for Pickup <span class="count">(%s)</span>', 'woocommerce')
           ));
       }
       
       /**
        * Adds "Ready for Pickup" to WooCommerce order status list.
        */
       public function mm_rfp_add_custom_order_status_to_list($order_statuses) {
           $order_statuses['wc-pickup-ready'] = __('Ready for Pickup', 'woocommerce');
           return $order_statuses;
       }

       /**
        * Adds "Ready for Pickup" to the bulk actions dropdown in admin.
        */
       public function mm_rfp_add_custom_order_status_bulk_actions($bulk_actions) {
           $bulk_actions['mark_wc-pickup-ready'] = __('Mark Ready for Pickup', 'woocommerce');
           return $bulk_actions;
       }

       /**
        * Handles bulk action for setting orders to "Ready for Pickup".
        */
       public function mm_rfp_handle_custom_order_status_bulk_action($redirect_to, $action, $post_ids) {
           if ($action === 'mark_wc-pickup-ready') {
               foreach ($post_ids as $post_id) {
                   $order = wc_get_order($post_id);
                   if ($order) {
                       $order->update_status('wc-pickup-ready', __('Order marked as Ready for Pickup.', 'woocommerce'),true);
                   }
               }
               $redirect_to = add_query_arg('bulk_pickup_ready', count($post_ids), $redirect_to);
           }
           return $redirect_to;
       }
       
       /**
        * Sends the "Ready for Pickup" email notification.
        */
       public function mm_rfp_send_ready_for_pickup_email($order_id, $order) {

           if (!$order || !is_a($order, 'WC_Order') || $order->get_status() !== 'pickup-ready') {
               return;
           }
           
           $mailer = WC()->mailer();
           $emails = $mailer->get_emails();
           
           if (!empty($emails['WC_Email_Ready_For_Pickup'])) {
               $emails['WC_Email_Ready_For_Pickup']->trigger($order_id);
           }
       }
       
       /**
        * Injects "Included in this Pickup" details into My Account Order Details
        */
       public function mm_rfp_add_order_details($order) {
           if ($order->get_status() !== 'pickup-ready') {
               return;
           }
           ?>
           <style>
               .pickup-container {
                   border: 1px solid #ddd;
                   padding: 15px;
                   margin-bottom: 20px;
                   background-color: #f9f9f9;
                   border-radius: 5px;
               }
               .pickup-header {
                   font-size: 18px;
                   font-weight: bold;
                   margin-bottom: 10px;
               }
               .pickup-items {
                   display: flex;
                   flex-wrap: wrap;
                   gap: 15px;
               }
               .pickup-item {
                   border: 1px solid #ddd;
                   padding: 10px;
                   background-color: #fff;
                   border-radius: 5px;
                   width: 200px;
                   text-align: center;
               }
               .pickup-item img {
                   max-width: 100px;
                   height: auto;
                   margin-bottom: 5px;
               }
           </style>

           <div class="pickup-container">
               <div class="pickup-header"><?php echo __('Included in this Pickup', 'managemore-connector'); ?></div>
               <div class="pickup-items">
                   <?php foreach ($order->get_items() as $item_id => $item) {
                       $product = $item->get_product();
                       $image_src = wp_get_attachment_image_src($product->get_image_id(), 'thumbnail');
                       $thumbnail = $image_src ? $image_src[0] : '';                   ?>
                   <div class="pickup-item">
                       <img src="<?php echo esc_url($thumbnail); ?>" alt="<?php echo esc_attr($product->get_name()); ?>">
                       <div><?php echo $product->get_name(); ?></div>
                       <div><strong>Qty:</strong> <?php echo $item->get_quantity(); ?></div>
                   </div>
                   <?php } ?>
               </div>
           </div>
           <?php
       }

       /**
        * Injects "Included in this Pickup" details into emails
        */
       public function mm_rfp_add_email_order_details($order) {
           if ($order->get_status() !== 'pickup-ready') {
               return;
           }
           ?>
           <style>
               .pickup-container {
                   border: 1px solid #ddd;
                   padding: 15px;
                   margin-bottom: 20px;
                   background-color: #f9f9f9;
                   border-radius: 5px;
               }
               .pickup-header {
                   font-size: 18px;
                   font-weight: bold;
                   margin-bottom: 10px;
               }
               .pickup-items {
                   display: flex;
                   flex-wrap: wrap;
                   gap: 15px;
               }
               .pickup-item {
                   border: 1px solid #ddd;
                   padding: 10px;
                   background-color: #fff;
                   border-radius: 5px;
                   width: 200px;
                   text-align: center;
               }
               .pickup-item img {
                   max-width: 100px;
                   height: auto;
                   margin-bottom: 5px;
               }
           </style>
           <div class="pickup-container">
               <div class="pickup-header"><?php echo __('Included in this Pickup', 'managemore-connector'); ?></div>
               <div class="pickup-items">
                   <?php foreach ($order->get_items() as $item_id => $item) {
                       $product = $item->get_product();
                       $image_src = wp_get_attachment_image_src($product->get_image_id(), 'thumbnail');
                       $thumbnail = $image_src ? $image_src[0] : '';                   ?>
                   <div class="pickup-item">
                       <img src="<?php echo esc_url($thumbnail); ?>" alt="<?php echo esc_attr($product->get_name()); ?>">
                       <div><?php echo $product->get_name(); ?></div>
                       <div><strong>Qty:</strong> <?php echo $item->get_quantity(); ?></div>
                   </div>
                   <?php } ?>
               </div>
           </div>
           <?php
       }


       /**
        * Registers the "Ready for Pickup" email class.
        */
       public function mm_register_ready_for_pickup_email($email_classes) {
           require_once MM_PLUGIN_PATH . 'includes/emails/class-wc-email-ready-for-pickup.php';
           $email_classes['WC_Email_Ready_For_Pickup'] = new WC_Email_Ready_For_Pickup();
           return $email_classes;
       }


       public function run() {
          //Custom Stauts Ready For Pickup (pickup-ready)
          // Register the custom order status
          add_action('init', array($this, 'mm_rfp_register_custom_order_status'));
          // Add the status to WooCommerce order statuses list
          add_filter('wc_order_statuses', array($this, 'mm_rfp_add_custom_order_status_to_list'));
          // Add "Ready for Pickup" status to bulk actions
          add_filter('bulk_actions-edit-shop_order', array($this, 'mm_rfp_add_custom_order_status_bulk_actions'));
          // Handle bulk actions
          add_filter('handle_bulk_actions-edit-shop_order', array($this, 'mm_rfp_handle_custom_order_status_bulk_action'), 10, 3);
          // Send "Ready for Pickup" email notification when order status is updated
          add_action('woocommerce_order_status_wc-pickup-ready', array($this, 'mm_rfp_send_ready_for_pickup_email'), 10, 2);

          // Hook to inject pickup details dynamically before WooCommerce's default order details
          add_action('woocommerce_order_details_before_order_table', array($this, 'mm_rfp_add_order_details'), 10, 1);
          // Hook to inject pickup details dynamically before email order details
          add_action('woocommerce_email_order_details', array($this, 'mm_rfp_add_email_order_details'), 10, 4);
          //Add the Ready for Pickup Email
          add_filter('woocommerce_email_classes', array($this, 'mm_register_ready_for_pickup_email'));
    	
       }


    }


}