<?php

final class MM_Object_Cache {
    const CACHE_GROUP = 'mm_connector';
    const DEFAULT_TTL = 2592000; // 30 days

    private static $debug_active = null;
    
    /**
     * Get a cached value by key.
     */
    public static function get(string $key) {
        $value = wp_cache_get($key, self::CACHE_GROUP);
        self::debug_log("GET key={$key} => " . json_encode($value));
        return $value;
    }

    /**
     * Set a cached value and optionally track it by one or more scopes.
     */
    public static function set(string $key, $value, array $scopes = [], int $ttl = null, string $index_group = 'default'): void {
        $ttl = $ttl ?? self::DEFAULT_TTL;
        wp_cache_set($key, $value, self::CACHE_GROUP, $ttl);
        self::debug_log("SET key={$key}, group=" . self::CACHE_GROUP);

        foreach ($scopes as $type => $id) {
            $index_key = self::build_index_key($type, $id);
            $group = self::get_index_group($index_group);
            $existing = wp_cache_get($index_key, $group);

            if (!is_array($existing)) {
                $existing = [];
            }

            if (!in_array($key, $existing, true)) {
                $existing[] = $key;
                wp_cache_set($index_key, $existing, $group, $ttl);
                self::debug_log("Indexed key under scope {$index_key} in group {$group}: " . json_encode($existing));
            }
        }
    }

    /**
     * Delete a cached value by key.
     */
    public static function delete(string $key): void {
        wp_cache_delete($key, self::CACHE_GROUP);
        self::debug_log("DELETE key={$key}, group=" . self::CACHE_GROUP);
    }

    /**
     * Delete all cache entries associated with a scope type/id.
     */
    public static function delete_by_scope(string $type, $id, string $index_group = 'default'): void {
        $index_key = self::build_index_key($type, $id);
        $group = self::get_index_group($index_group);
        $keys = wp_cache_get($index_key, $group);

        self::debug_log("DELETE_BY_SCOPE index_key={$index_key}, group={$group}, keys=" . json_encode($keys));

        if (is_array($keys)) {
            foreach ($keys as $key) {
                wp_cache_delete($key, self::CACHE_GROUP);
                self::debug_log(" - Deleted scoped key: {$key}");
            }
            wp_cache_delete($index_key, $group);
            self::debug_log(" - Deleted index key: {$index_key}");
        } else {
            self::debug_log(" - No keys found for scope: {$index_key}");
        }
    }

    /**
     * Build the index key used to track grouped cache keys.
     */
    private static function build_index_key(string $type, $id): string {
        return "{$type}_{$id}";
    }

    /**
     * Get the full index group name.
     */
    private static function get_index_group(string $group_type): string {
        return "mm_connector_index_{$group_type}";
    }

    /**
     * Optional debug logger.
     */
    private static function debug_log($message) {
        if (self::$debug_active === null) {
           self::$debug_active = MM_Debug_Mode::is_active(MM_Debug_Mode::CACHE);
        }
        if (self::$debug_active) {
           MM_Connector_Functions::debug_log($message, MM_Debug_Mode::CACHE);
        }
    }
}
