<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * ManageMore Payment Gateway
 *
 * Provides a ManageMore Payment Gateway
 *
 * @class 		WC_Gateway_ManageMore_OnAccount
 * @extends		WC_Payment_Gateway_CC
 * @version		4.0.0
 * @package		WooCommerce/Classes/Payment
 * @author 		Intellisoft Solutions, Inc.
 */
class WC_Gateway_ManageMore_Onaccount extends WC_Payment_Gateway_CC {
	
	private $instructions;
	private $restrict_use;
	private $credit_limit_message;


    /**
     * Constructor for the gateway.
     */
	public function __construct() {
		$this->id                 = 'managemore_onaccount';
		$this->icon               = null;
		$this->has_fields         = false;
		// Supports the default credit card form
		$this->supports = array( 'product' );
		$this->method_title       = __( 'ManageMore On Account', 'managemore-connector' );
		$this->method_description = __( 'Allows orders to be placed on account for ManageMore Customers.', 'managemore-connector' );

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

      // Turn these settings into variables we can use
		foreach ( $this->settings as $setting_key => $value ) {
			$this->$setting_key = $value;
		}
		// Define user set variables
		$this->title        = $this->get_option( 'title' );
		$this->description  = $this->get_option( 'description' );
		$this->instructions = $this->get_option( 'instructions', $this->description );


    //Save Settings
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    
  }

    /**
     * Initialise Gateway Settings Form Fields
     */
    public function init_form_fields() {
      
    	$this->form_fields = array(
			'enabled' => array(
				'title'   => __( 'Enable/Disable', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable On Account Orders for ManageMore', 'managemore-connector' ),
				'default' => 'no'
			),
			'title' => array(
				'title'       => __( 'Title', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
				'default'     => __( 'On Account', 'woocommerce' ),
				'desc_tip'    => true,
			),
        'restrict_use' => array(
            'title'   => __( 'Restrict Use', 'managemore-connector' ),
            'type'    => 'checkbox',
            'label'   => __( 'Restrict use to synchronized ManageMore customers with account privileges and sufficient credit limit', 'managemore-connector' ),
            'default' => 'no'
        ),
			'credit_limit_message' => array(
				'title'       => __( 'Credit Limit Message', 'managemore-connector' ),
				'type'        => 'textarea',
				'description' => __( 'This controls the error message that user sees when the credit limit is exceeded. You can use placeholders {balance} {pending_orders} and {credit_limit} in the message.', 'woocommerce' ),
				'default'     => __( 'Sorry, your order total will exceed your available credit limit of {credit_limit}. Your current balance is {balance}. You have pending orders in the amount of {pending_orders}.', 'managemore-connector' ),
				'desc_tip'    => true,
			),
      );
    }


 



  /**
  * Process the payment and return the result
  * Note: for this payment method, no payment is processed.  We just do some checks, 
  *       like whether credit limit has been exceeded.
  *
  * @param int $order_id
  * @return array
  */
	public function process_payment( $order_id ) {
      global $woocommerce;

      
      //Get Order
 	   $order = new WC_Order( $order_id );

      // Check if restrict_use setting is enabled
      // Added 4.0.0
      if ($this->restrict_use == 'yes') {
          // Get customer ID from order
          $customer_id = $order->get_customer_id();

          // Get customer's balance and credit limit from user meta
          $balance = get_user_meta($customer_id, 'mm_field_B', true);
          $credit_limit = get_user_meta($customer_id, 'mm_field_CLMT', true);
          $pending_orders = $this->get_processing_order_total_for_user($customer_id);
          
          // Check if credit limit is set and is not zero
          if ($credit_limit && $credit_limit > 0) {
              // If existing balance plus total order amount plus pending orders exceeds credit limit, don't process the order and return an error
              if ($order->get_total()+$balance+$pending_orders > $credit_limit) {
              	   $credit_limit_message = $this->process_credit_limit_message($balance,$credit_limit,$pending_orders);
                  wc_add_notice( $credit_limit_message, 'error' );
                  return;
              }
          }
      }
		
      //Call Gateway (there is none)
      unset($response);
      
     // Mark order as Paid
     // Added 03.26.2020, so order is marked as Processing and not Pending Payment
     $order->payment_complete();
   			 
   			 
	  // Empty the cart (Very important step)
	  $woocommerce->cart->empty_cart();


	  // Redirect to thank you page
     return array(
 			'result'   => 'success',
	   	'redirect' => $this->get_return_url( $order ),
	  );
			    
			    
	}


   /**
    * Replace placeholders in credit limit message with actual values.
    * 
    *
    * Added 4.0.0
    *
    * @param int $customer_id The ID of the customer to retrieve balance and credit limit for.
    * @return string The processed message.
    */
   private function process_credit_limit_message($balance, $credit_limit, $pending_orders) {
 
       // Format balance and credit limit as two decimal numbers
       // If balance or credit limit are not set, display as 0.00 or N/A
       $balance = isset($balance) ? number_format((float)$balance, 2, '.', '') : '0.00';
       $credit_limit = (isset($credit_limit) && $credit_limit > 0 ) ? number_format((float)$credit_limit, 2, '.', '') : 'N/A';
       $pending_orders = isset($pending_orders) ? number_format((float)$pending_orders, 2, '.', '') : '0.00';


       // Replace {balance} and {credit_limit} placeholders with actual values
       $message = str_replace('{balance}', $balance, $this->credit_limit_message);
       $message = str_replace('{credit_limit}', $credit_limit, $message);
       $message = str_replace('{pending_orders}', $pending_orders, $message);

       return $message;
   }

   /**
    * Fetches and sums all 'processing' orders for a given user ID.
    *
    * This function uses the WC_Order_Query class to fetch all orders for a 
    * specific user that are in 'processing' status, and are On Account. It then sums the totals 
    * of these orders and returns the total.
    *
    * Note: the total includes taxes and shipping. If these should be excluded,
    * the total calculation will need to be adjusted accordingly.
    *
    * Added 4.0.0
    *
    * @param int $user_id The ID of the user to fetch orders for.
    * @return float The total of all 'processing' orders for the user.
    */
   private function get_processing_order_total_for_user($user_id) {
       $total = 0.0;

       $order_query = new WC_Order_Query(array(
           'customer_id' => $user_id,
           'status' => 'processing',
           'payment_method' => 'managemore_onaccount',
           'return' => 'ids',  // We only need IDs
       ));

       $order_ids = $order_query->get_orders();

       foreach ($order_ids as $order_id) {
           $order = wc_get_order($order_id);
           $total += $order->get_total();
       }

       return $total;
   }

   public function get_restrict_use() {
       return isset($this->restrict_use) ? $this->restrict_use : 'no';
   }
  
}
