<?php
/*
Database Related Functions
*/


if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

//Check database upon install
function mm_connector_db_install() {

	global $wpdb;
	global $mm_connector_db_version;

 /*
  * Flush rewrite rules to ensure our new endpoint is registered and accessible. 
  * This is done on plugin activation because WordPress caches rewrite rules and 
  * only updates them when the permalinks settings are saved, or when 
  * flush_rewrite_rules() is called. By calling this function on activation, we 
  * ensure that our custom endpoint is recognized and properly handled by WordPress.
  *
  * We're creating a custom endpoint (account-status) in the WooCommerce My Account page 
  * for users to check their account status. This custom endpoint needs to be added 
  * into WordPress's rewrite rules so that when a user navigates to 
  * 'myaccount/account-status', WordPress can correctly interpret and handle this URL.
  *
  * However, WordPress caches its rewrite rules and only updates them when the permalinks 
  * settings are saved or when flush_rewrite_rules() is explicitly called. As our new 
  * endpoint won't be recognized until the rewrite rules are updated, we need to ensure 
  * that the rules are flushed upon plugin activation.
  *
  * Thus, on plugin activation, we're calling flush_rewrite_rules() to immediately update 
  * WordPress's cached rewrite rules. This ensures our custom endpoint 'account-status' 
  * is registered and accessible right from the plugin activation.
  *
  * Note: This has nothing to to with database.  the only reason its here is that this is
  *       called on install.
  *
  */
   flush_rewrite_rules();


   $installed_db_ver = get_option( 'mm_connector_db_version' );
   
   //Notes
   // db version 1.3 (plugin version >= 2.1.5)
   // 1) Increased indirect_tax_rate to DECIMAL(15,4) from 15,2 due to rounding problems

   if ( $installed_db_ver != $mm_connector_db_version ) {

   	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

   	$table_name = $wpdb->prefix . 'mm_customer_price';
   	$charset_collate = $wpdb->get_charset_collate();
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	user_id INT NOT NULL,
   	product_id INT NOT NULL,
   	method tinyint not NULL,
   	amount DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	start_date varchar(10) DEFAULT '',
   	end_date   varchar(10) DEFAULT '',
   	PRIMARY KEY  (id),
   	INDEX user (user_id,product_id),
   	INDEX product (product_id,user_id)
   	) $charset_collate;";
   	dbDelta( $sql );

   	$table_name = $wpdb->prefix . 'mm_indirect_tax';
   	$charset_collate = $wpdb->get_charset_collate();
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	indirect_tax_type INT NOT NULL,
   	indirect_tax_state varchar(3) DEFAULT '',
   	indirect_tax_calc_method tinyint not NULL,
   	indirect_tax_rate DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	PRIMARY KEY  (id),
   	INDEX state (indirect_tax_type,indirect_tax_state)
   	) $charset_collate;";
   	dbDelta( $sql );

    //Indirec Tax Jurisdiction table, introduced plug in version 5.0.1 (db version 1.4)
   	$table_name = $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
   	$charset_collate = $wpdb->get_charset_collate();
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	indirect_tax_type INT NOT NULL,
   	indirect_tax_code varchar(6) DEFAULT '',
   	indirect_tax_calc_method_j1 tinyint not NULL,
   	indirect_tax_rate_j1 DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	indirect_tax_calc_method_j2 tinyint not NULL,
   	indirect_tax_rate_j2 DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	PRIMARY KEY  (id),
   	INDEX code (indirect_tax_type,indirect_tax_code)
   	) $charset_collate;";
   	dbDelta( $sql );

   }

   //Check if old data needs to be converted - Before version 2.0.0, we stored data in custom posts/post meta
   //But this was inefficient, especially if you have thousands of records
   //Note: This code was removed, because noone should have the old plug-in at this point
   /*
   if (version_compare( $installed_ver,$mm_connector_db_version,'<')){

      $table_name = $wpdb->prefix . 'mm_customer_price';
      $customer_price_count = $wpdb->get_var( "SELECT COUNT(*) FROM ".$table_name);
      if ($customer_price_count == 0){
          $posts = get_posts(
                         array(
                               'post_type'  => 'mm-customer-price',
                               'post_status' => 'publish',
                               'numberposts' => -1
                              )
                        );
          if ($posts){
               //If we find an old customer price, convert and remove
               foreach($posts as $post){
               	  $customer_price_id = $post->ID;
               	  $user_id = get_post_meta($customer_price_id,'user_id',true);
               	  $product_id = get_post_meta($customer_price_id,'product_id',true);
              	     $method = get_post_meta($customer_price_id,'method',true);
               	  $amount = get_post_meta($customer_price_id,'amount',true);
              	     $start_date = get_post_meta($customer_price_id,'start_date',true);
              	     $end_date = get_post_meta($customer_price_id,'end_date',true);


                    $wpdb->insert( 
                                $table_name, 
                                array( 
                                     'user_id' => $user_id, 
                                     'product_id' => $product_id, 
                                     'method' => $method, 
                                     'amount' => $amount, 
                                     'start_date' => $start_date, 
                                     'end_date' => $end_date 
                                     ) 
                                 );
                                 
                    //delete the custom post           
                    wp_delete_post($post->ID, true ); //skip trash
               }
          }
      }
      //Check if old Indirect Tax Rates
      $table_name = $wpdb->prefix . 'mm_indirect_tax';
      $indirect_tax_count = $wpdb->get_var( "SELECT COUNT(*) FROM ".$table_name);
      if ($indirect_tax_count == 0){
          $posts = get_posts(
                         array(
                               'post_type'  => 'mm-indirect-tax',
                               'post_status' => 'publish',
                               'numberposts' => -1
                              )
                        );
          if ($posts){
               //If we find an old customer price, convert and remove
               foreach($posts as $post){
               	  $indirect_tax_post_id = $post->ID;
               	  $indirect_tax_type = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_type',true);
               	  $indirect_tax_state = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_state',true);
              	     $indirect_tax_rate = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_rate',true);
               	  $indirect_tax_calc_method = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_calc_method',true);


                    $wpdb->insert( 
                                $table_name, 
                                array( 
                                     'indirect_tax_type' => $indirect_tax_type, 
                                     'indirect_tax_state' => $indirect_tax_state, 
                                     'indirect_tax_rate' => $indirect_tax_rate, 
                                     'indirect_tax_calc_method' => $indirect_tax_calc_method
                                     ) 
                                 );
                                 
                    //delete the custom post           
                    wp_delete_post($post->ID, true ); //skip trash
               }
          }


      }



   	
   }
   */
   
   
  
   if( $installed_db_ver == '' ){
   	add_option( 'mm_connector_db_version', $mm_connector_db_version );
   }
   else {
   	update_option( 'mm_connector_db_version', $mm_connector_db_version );
   }
         
}




//Check if Update of Database needed when updating plugin
//IMPORTANT: This function fires every page load, so be sure any code in this procedure is considering version changes only
function mm_connector_update_db_check() {
 	  global $wpdb;
    global $mm_connector_db_version;

    //Check whether database needs updating
    if ( get_site_option( 'mm_connector_db_version' ) != $mm_connector_db_version ) {
        mm_connector_db_install();
    }
    
    //Perform other updates when plug-in is installed
    $last_plugin_version = get_option('mm_connector_version', '3.0.3');  //'3.0.3' is default if no version is stored yet


    //The stored version is less than the 4.0.0, schedule the job to upgrade user dates from mm/dd/yyyy to yyyy/mm/dd
    if (version_compare($last_plugin_version, '4.0.0', '<')) {
    	  //WP-Cron is not used here, since we rely on Woo, we use the action scheduler, like they do. (they install it)
        wp_schedule_single_event(time(), 'mm_connector_upgrade_dates_event');  //WP-Cron method
        //Check for the action scheduler funtion before using it (in case it isnt installed)
        //If not, the job will fail.  But, since we rely heavily on Woo, this is extremely unlikely
        if ( function_exists('as_schedule_single_action') ) {
           as_schedule_single_action( time(), 'mm_connector_upgrade_dates_event' );
        }
    }
    //Update the stored version
    update_option('mm_connector_version', MM_CURRENT_VERSION);

    
}



/**
 * Function mm_connector_reformat_dates
 * 
 * This function is intended to reformat date fields associated with each user in the system.
 * Specifically, it focuses on the 'mm_field_L1X', 'mm_field_L2X', 'mm_field_L3X', and 
 * 'mm_field_LPD' fields. It converts date values from 'mm/dd/yyyy' format into 'yyyy/mm/dd'
 * format. This standardization of date format is necessary to ensure compatibility with
 * Ultimate Member plugin date standards and correct functioning of date-based operations
 * within the plugin. The function is designed to be run as a cron job or similar scheduled 
 * task to continuously ensure all date values remain correctly formatted even as new data 
 * is added or existing data is updated.
 *
 */
function mm_connector_reformat_dates() {
	
	 
    // Get all users.
    $users = get_users();

    foreach ($users as $user) {
        $fields = ['mm_field_L1X', 'mm_field_L2X', 'mm_field_L3X', 'mm_field_LPD'];

        foreach ($fields as $field) {
            $date = get_user_meta($user->ID, $field, true);

            // If date exists and format is mm/dd/yyyy.
            if ($date && substr($date, 2, 1) === '/') {
                // Convert date from mm/dd/yyyy to yyyy-mm-dd for use in strtotime
                $date_parts = explode('/', $date);
                if (count($date_parts) === 3) {
                    $new_date = $date_parts[2] . '/' . $date_parts[0] . '/' . $date_parts[1];
                    update_user_meta($user->ID, $field, $new_date);
                }
            }
        }
    }
    

}



?>