<?php
/*
Database Related Functions
*/


if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

//Check database upon install
function mm_connector_db_install() {

	global $wpdb;
	global $mm_connector_db_version;

 /*
  * Flush rewrite rules to ensure our new endpoint is registered and accessible. 
  * This is done on plugin activation because WordPress caches rewrite rules and 
  * only updates them when the permalinks settings are saved, or when 
  * flush_rewrite_rules() is called. By calling this function on activation, we 
  * ensure that our custom endpoint is recognized and properly handled by WordPress.
  *
  * We're creating a custom endpoint (account-status) in the WooCommerce My Account page 
  * for users to check their account status. This custom endpoint needs to be added 
  * into WordPress's rewrite rules so that when a user navigates to 
  * 'myaccount/account-status', WordPress can correctly interpret and handle this URL.
  *
  * However, WordPress caches its rewrite rules and only updates them when the permalinks 
  * settings are saved or when flush_rewrite_rules() is explicitly called. As our new 
  * endpoint won't be recognized until the rewrite rules are updated, we need to ensure 
  * that the rules are flushed upon plugin activation.
  *
  * Thus, on plugin activation, we're calling flush_rewrite_rules() to immediately update 
  * WordPress's cached rewrite rules. This ensures our custom endpoint 'account-status' 
  * is registered and accessible right from the plugin activation.
  *
  * Note: This has nothing to to with database.  the only reason its here is that this is
  *       called on install.
  *
  */
   flush_rewrite_rules();


   $installed_db_ver = get_option( 'mm_connector_db_version' );
   //Notes
   // db version 1.3 (plugin version >= 2.1.5)
   // 1) Increased indirect_tax_rate to DECIMAL(15,4) from 15,2 due to rounding problems

   if ( $installed_db_ver != $mm_connector_db_version ) {
    error_log('ManageMore Connector: (db install) Upgrading Database - installed_db_ver:'.$installed_db_ver.' mm_connector_db_version:'.$mm_connector_db_version);
   	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
   	$charset_collate = $wpdb->get_charset_collate();

   	$table_name = $wpdb->prefix . 'mm_customer_price';
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	user_id INT NOT NULL,
   	product_id INT NOT NULL,
   	method tinyint not NULL,
   	amount DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	start_date varchar(10) DEFAULT '',
   	end_date   varchar(10) DEFAULT '',
   	PRIMARY KEY  (id),
   	INDEX user (user_id,product_id),
   	INDEX product (product_id,user_id)
   	) $charset_collate;";
   	dbDelta( $sql );

   	$table_name = $wpdb->prefix . 'mm_indirect_tax';
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	indirect_tax_type INT NOT NULL,
   	indirect_tax_state varchar(3) DEFAULT '',
   	indirect_tax_calc_method tinyint not NULL,
   	indirect_tax_rate DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	min_max_calc_method tinyint DEFAULT 4 NOT NULL,
   	max_amount_to_tax DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	min_amount_to_tax DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	PRIMARY KEY  (id),
   	INDEX state (indirect_tax_type,indirect_tax_state)
   	) $charset_collate;";
   	dbDelta( $sql );

    //Indirect Tax Jurisdiction table, introduced plug in version 5.0.1 (db version 1.4)
   	$table_name = $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
   	$sql = "CREATE TABLE $table_name (
   	id INT NOT NULL AUTO_INCREMENT,
   	indirect_tax_type INT NOT NULL,
   	indirect_tax_code varchar(6) DEFAULT '',
   	indirect_tax_calc_method_j1 tinyint not NULL,
   	indirect_tax_rate_j1 DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	indirect_tax_calc_method_j2 tinyint not NULL,
   	indirect_tax_rate_j2 DECIMAL(15,4) DEFAULT 0.0000 NOT NULL,
   	min_max_calc_method_j1 tinyint DEFAULT 4 NOT NULL,
   	min_max_calc_method_j2 tinyint DEFAULT 4 NOT NULL,
   	max_amount_to_tax_j1 DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	min_amount_to_tax_j1 DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	max_amount_to_tax_j2 DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	min_amount_to_tax_j2 DECIMAL(15,2) DEFAULT 0.00 NOT NULL,
   	PRIMARY KEY  (id),
   	INDEX code (indirect_tax_type,indirect_tax_code)
   	) $charset_collate;";
   	dbDelta( $sql );

    //Trade Class Restrictions
    $table_name = $wpdb->prefix . 'mm_trade_class_restrictions'; 
    $sql = "CREATE TABLE IF NOT EXISTS $table_name (
       trade_class_restrict_id INT UNSIGNED NOT NULL,
       state CHAR(3) NOT NULL,
       trade_class_id INT UNSIGNED NOT NULL,
       PRIMARY KEY (trade_class_restrict_id, state, trade_class_id),
       INDEX idx_state_tradeclass (state, trade_class_id)
       ) $charset_collate;";
  	dbDelta( $sql );

    // Trade Class Table (Descriptions for trade_class_id)
    $table_name = $wpdb->prefix . 'mm_trade_class'; 
    $sql = "CREATE TABLE IF NOT EXISTS $table_name (
        trade_class_id INT UNSIGNED NOT NULL,
        trade_class_name VARCHAR(255) NOT NULL UNIQUE,
        description TEXT NULL,
        PRIMARY KEY (trade_class_id),
        INDEX idx_trade_class_name (trade_class_name)
    ) $charset_collate;";
  	dbDelta( $sql );

    // Trade Class Restrict Names Table (Descriptions for trade_class_restrict_id)
    $table_name = $wpdb->prefix . 'mm_trade_class_restrict_names'; 
    $sql = "CREATE TABLE IF NOT EXISTS $table_name (
        trade_class_restrict_id INT UNSIGNED NOT NULL,
        restriction_name VARCHAR(255) NOT NULL,
        PRIMARY KEY (trade_class_restrict_id),
        INDEX idx_restriction_name (restriction_name)
    ) $charset_collate;";
  	dbDelta( $sql );



   }

   //Check if old data needs to be converted - Before version 2.0.0, we stored data in custom posts/post meta
   //But this was inefficient, especially if you have thousands of records
   if (version_compare( $installed_db_ver,$mm_connector_db_version,'<')){

      error_log('ManageMore Connector: (db install) Converting Database - installed_db_ver:'.$installed_db_ver.' mm_connector_db_version:'.$mm_connector_db_version);

      //Note: This code was removed, because noone should have the old plug-in at this point
      /*
      $table_name = $wpdb->prefix . 'mm_customer_price';
      $customer_price_count = $wpdb->get_var( "SELECT COUNT(*) FROM ".$table_name);
      if ($customer_price_count == 0){
          $posts = get_posts(
                         array(
                               'post_type'  => 'mm-customer-price',
                               'post_status' => 'publish',
                               'numberposts' => -1
                              )
                        );
          if ($posts){
               //If we find an old customer price, convert and remove
               foreach($posts as $post){
               	  $customer_price_id = $post->ID;
               	  $user_id = get_post_meta($customer_price_id,'user_id',true);
               	  $product_id = get_post_meta($customer_price_id,'product_id',true);
              	     $method = get_post_meta($customer_price_id,'method',true);
               	  $amount = get_post_meta($customer_price_id,'amount',true);
              	     $start_date = get_post_meta($customer_price_id,'start_date',true);
              	     $end_date = get_post_meta($customer_price_id,'end_date',true);


                    $wpdb->insert( 
                                $table_name, 
                                array( 
                                     'user_id' => $user_id, 
                                     'product_id' => $product_id, 
                                     'method' => $method, 
                                     'amount' => $amount, 
                                     'start_date' => $start_date, 
                                     'end_date' => $end_date 
                                     ) 
                                 );
                                 
                    //delete the custom post           
                    wp_delete_post($post->ID, true ); //skip trash
               }
          }
      }
      //Check if old Indirect Tax Rates
      $table_name = $wpdb->prefix . 'mm_indirect_tax';
      $indirect_tax_count = $wpdb->get_var( "SELECT COUNT(*) FROM ".$table_name);
      if ($indirect_tax_count == 0){
          $posts = get_posts(
                         array(
                               'post_type'  => 'mm-indirect-tax',
                               'post_status' => 'publish',
                               'numberposts' => -1
                              )
                        );
          if ($posts){
               //If we find an old customer price, convert and remove
               foreach($posts as $post){
               	  $indirect_tax_post_id = $post->ID;
               	  $indirect_tax_type = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_type',true);
               	  $indirect_tax_state = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_state',true);
              	     $indirect_tax_rate = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_rate',true);
               	  $indirect_tax_calc_method = get_post_meta($indirect_tax_post_id,'mm_indirect_tax_calc_method',true);


                    $wpdb->insert( 
                                $table_name, 
                                array( 
                                     'indirect_tax_type' => $indirect_tax_type, 
                                     'indirect_tax_state' => $indirect_tax_state, 
                                     'indirect_tax_rate' => $indirect_tax_rate, 
                                     'indirect_tax_calc_method' => $indirect_tax_calc_method
                                     ) 
                                 );
                                 
                    //delete the custom post           
                    wp_delete_post($post->ID, true ); //skip trash
               }
          }


      }
      */


   	
   }
   // Upgrade Code =============================================================
   
  
   if( $installed_db_ver == '' ){
   	add_option( 'mm_connector_db_version', $mm_connector_db_version );
   }
   else {
   	update_option( 'mm_connector_db_version', $mm_connector_db_version );
   }
         
}




//Check if Update of Database needed when updating plugin
//IMPORTANT: This function fires every page load, so be sure any code in this procedure is considering version changes only
function mm_connector_update_db_check() {
 	  global $wpdb;
    global $mm_connector_db_version;

    //Check whether database needs updating
    if ( get_site_option( 'mm_connector_db_version' ) != $mm_connector_db_version ) {
        mm_connector_db_install();
    }
    
    //Perform other updates when plug-in is installed
    $last_plugin_version = get_option('mm_connector_version', '3.0.3');  //'3.0.3' is default if no version is stored yet


    //The stored version is less than the 4.0.0, schedule the job to upgrade user dates from mm/dd/yyyy to yyyy/mm/dd
    if (version_compare($last_plugin_version, '4.0.0', '<')) {
    	  //WP-Cron is not used here, since we rely on Woo, we use the action scheduler, like they do. (they install it)
        //Check for the action scheduler funtion before using it (in case it isnt installed)
        //If not, the job will fail.  But, since we rely heavily on Woo, this is extremely unlikely
        if ( function_exists('as_schedule_single_action') ) {
           as_schedule_single_action( time(), 'mm_connector_upgrade_dates_event' );
        }
        else {
           wp_schedule_single_event(time(), 'mm_connector_upgrade_dates_event');  //WP-Cron method
        }
    }
    
    //Check if UPC Codes Need Converting
    if (get_option('mm_upc_upgrade_complete', 0) != 1){
    	//Add notice with link to run process
    	add_action('admin_notices', 'mm_upc_upgrade_check_admin_notice');
    	// Hooks to process the upgrade when the link is clicked
      add_action('admin_init', 'mm_upc_upgrade_process');
      add_action('mm_connector_upc_upgrade_action', 'mm_connector_upc_upgrade_function');
    }

    if(get_option('mm_upc_upgrade_notice', 0) == 1){
      update_option('mm_upc_upgrade_notice', 0);
	    add_action('admin_notices', function() { echo '<div class="notice notice-success is-dismissible">
    	                                             <p><strong>ManageMore Connector:</strong> UPC conversion complete!</p></div>'; 
    	                                     }
      );

    }
    //Update the stored version
    update_option('mm_connector_version', MM_CURRENT_VERSION);

    
}

function mm_upc_upgrade_check_admin_notice() {
   $url = esc_url(add_query_arg('mm_upc_upgrade_action', 'run', $_SERVER['REQUEST_URI']));
   echo '<div class="notice notice-warning is-dismissible">
         <p><strong>ManageMore Connector:</strong> A database conversion is required for UPC processing.<br>
         You should perform this conversion ONLY after updating ManageMore Business Software to the latest build.<br><br>
         <a href="' . $url . '" class="button button-primary">Run Conversion</a></p>
         </div>';
}

function mm_upc_upgrade_process() {
    if (isset($_GET['mm_upc_upgrade_action']) && $_GET['mm_upc_upgrade_action'] === 'run') {
        // Run the database upgrade process here
        do_action('mm_connector_upc_upgrade_action');


        // Redirect to remove query param and avoid re-triggering on refresh
        wp_safe_redirect(remove_query_arg('mm_upc_upgrade_action'));
        exit;
    }
}

function mm_connector_upc_upgrade_function() {
    // Your upgrade logic goes here
    // Example: Update database schema, migrate data, etc.
    global $wpdb;

    error_log('New Function');

    error_log('ManageMore Connector: (db install) Converting UPC Codes.');

    $table_name = $wpdb->prefix . 'postmeta';

   	$sql = "INSERT INTO $table_name (post_id, meta_key, meta_value)
                SELECT pm.post_id, '_global_unique_id', pm.meta_value
                FROM $table_name pm
                WHERE pm.meta_key = 'mm_product_upc'
                ON DUPLICATE KEY UPDATE meta_value = VALUES(meta_value);";
    $result = $wpdb->query($sql);
                
    if ($result === false) {
       error_log("ManageMore Connector: Error updating UPC in _global_unique_id: " . $wpdb->last_error);
    } else {
       error_log("ManageMore Connector: Converting UPC Codes executed successfully. Rows affected: " . $wpdb->rows_affected);
       // Set the flag to indicate completion, and to trigger notice
       update_option('mm_upc_upgrade_complete', 1);
       update_option('mm_upc_upgrade_notice', 1);
    }
    

    //$sql = "DELETE FROM $table_name WHERE meta_key = 'mm_product_upc';";
    //$wpdb->query($sql);
}



/**
 * Function mm_connector_reformat_dates
 * 
 * This function is intended to reformat date fields associated with each user in the system.
 * Specifically, it focuses on the 'mm_field_L1X', 'mm_field_L2X', 'mm_field_L3X', and 
 * 'mm_field_LPD' fields. It converts date values from 'mm/dd/yyyy' format into 'yyyy/mm/dd'
 * format. This standardization of date format is necessary to ensure compatibility with
 * Ultimate Member plugin date standards and correct functioning of date-based operations
 * within the plugin. The function is designed to be run as a cron job or similar scheduled 
 * task to continuously ensure all date values remain correctly formatted even as new data 
 * is added or existing data is updated.
 *
 */
function mm_connector_reformat_dates() {
	
	 
    // Get all users.
    $users = get_users();

    foreach ($users as $user) {
        $fields = ['mm_field_L1X', 'mm_field_L2X', 'mm_field_L3X', 'mm_field_LPD'];

        foreach ($fields as $field) {
            $date = get_user_meta($user->ID, $field, true);

            // If date exists and format is mm/dd/yyyy.
            if ($date && substr($date, 2, 1) === '/') {
                // Convert date from mm/dd/yyyy to yyyy-mm-dd for use in strtotime
                $date_parts = explode('/', $date);
                if (count($date_parts) === 3) {
                    $new_date = $date_parts[2] . '/' . $date_parts[0] . '/' . $date_parts[1];
                    update_user_meta($user->ID, $field, $new_date);
                }
            }
        }
    }
    

}



?>